import os
import json
import platform
import time
from typing import Dict, Optional, Any, List
from pathlib import Path
from contextlib import contextmanager
from datetime import datetime

# 跨平台文件锁导入
try:
    import fcntl
except ImportError:
    fcntl = None

try:
    import msvcrt
except ImportError:
    msvcrt = None


RAGS_JSON = Path.home() / ".auto-coder" / "keys" / "rags_config.json"


class RAGConfigManager:
    """RAG配置管理器，支持并发安全的增删改查操作"""

    def __init__(self, config_path: str = str(RAGS_JSON)):
        self.config_path = config_path
        self._ensure_config_dir()

    def _ensure_config_dir(self):
        """确保配置目录存在"""
        config_path = Path(self.config_path)
        config_dir = config_path.parent
        config_dir.mkdir(parents=True, exist_ok=True)

    @contextmanager
    def _file_lock(self, mode="r"):
        """
        文件锁上下文管理器，确保并发安全
        支持Unix (fcntl) 和 Windows (msvcrt) 系统
        """
        # 确保文件存在
        config_path = Path(self.config_path)
        if not config_path.exists():
            config_path.parent.mkdir(parents=True, exist_ok=True)
            with open(config_path, "w") as f:
                json.dump({}, f)

        file_handle = open(config_path, mode)
        lock_acquired = False

        try:
            # 尝试获取文件锁
            if platform.system() != "Windows":
                # Unix系统使用fcntl
                if fcntl is not None:
                    if "w" in mode or "a" in mode or "+" in mode:
                        fcntl.flock(file_handle.fileno(), fcntl.LOCK_EX)  # 独占锁
                    else:
                        fcntl.flock(file_handle.fileno(), fcntl.LOCK_SH)  # 共享锁
                    lock_acquired = True
            else:
                # Windows系统的文件锁实现
                if msvcrt is not None:
                    try:
                        # 在Windows上，我们使用独占锁来简化实现
                        # msvcrt.locking 参数: fd, mode, nbytes
                        # LK_NBLCK: 非阻塞独占锁
                        # LK_LOCK: 阻塞独占锁
                        max_attempts = 10
                        for attempt in range(max_attempts):
                            try:
                                # 尝试锁定文件的第一个字节
                                file_handle.seek(0)
                                msvcrt.locking(file_handle.fileno(), msvcrt.LK_NBLCK, 1)
                                lock_acquired = True
                                break
                            except IOError:
                                # 如果锁被占用，等待一小段时间后重试
                                if attempt < max_attempts - 1:
                                    time.sleep(0.1)
                                else:
                                    # 最后一次尝试使用阻塞锁
                                    try:
                                        file_handle.seek(0)
                                        msvcrt.locking(
                                            file_handle.fileno(), msvcrt.LK_LOCK, 1
                                        )
                                        lock_acquired = True
                                    except IOError:
                                        # 如果仍然失败，继续执行但不使用锁
                                        pass
                    except Exception:
                        # 如果出现任何异常，继续执行但不使用锁
                        pass

            # 重置文件位置到开始
            file_handle.seek(0)
            yield file_handle

        finally:
            try:
                # 释放文件锁
                if lock_acquired:
                    if platform.system() != "Windows":
                        if fcntl is not None:
                            fcntl.flock(file_handle.fileno(), fcntl.LOCK_UN)
                    else:
                        if msvcrt is not None:
                            try:
                                file_handle.seek(0)
                                msvcrt.locking(file_handle.fileno(), msvcrt.LK_UNLCK, 1)
                            except IOError:
                                # 忽略解锁错误
                                pass
            except Exception:
                # 忽略释放锁时的错误
                pass
            finally:
                file_handle.close()

    def _load_config(self) -> Dict[str, Any]:
        """
        加载配置文件
        """
        try:
            with self._file_lock("r") as f:
                content = f.read().strip()
                if not content:
                    return {}
                return json.loads(content)
        except (FileNotFoundError, json.JSONDecodeError):
            return {}

    def _save_config(self, config: Dict[str, Any]):
        """
        保存配置文件
        """
        with self._file_lock("w") as f:
            json.dump(config, f, indent=2, ensure_ascii=False)

    def create(self, name: str, config: Dict[str, Any]) -> bool:
        """
        创建新的RAG服务配置

        Args:
            name: RAG服务名称
            config: 配置字典

        Returns:
            bool: 创建成功返回True，如果名称已存在返回False
        """
        all_configs = self._load_config()

        if name in all_configs:
            return False

        # 确保配置包含必要的字段
        config["name"] = name
        if "status" not in config:
            config["status"] = "stopped"
        if "created_at" not in config:
            config["created_at"] = datetime.now().isoformat()
        config["updated_at"] = datetime.now().isoformat()

        all_configs[name] = config
        self._save_config(all_configs)
        return True

    def read(self, name: Optional[str] = None) -> Optional[Dict[str, Any]]:
        """
        读取RAG服务配置

        Args:
            name: RAG服务名称，如果为None则返回所有配置

        Returns:
            Dict: 配置字典，如果name不存在或为None且无配置则返回None
        """
        all_configs = self._load_config()

        if name is None:
            return all_configs if all_configs else None

        return all_configs.get(name)

    def update(self, name: str, config: Dict[str, Any]) -> bool:
        """
        更新RAG服务配置

        Args:
            name: RAG服务名称
            config: 新的配置字典（会与现有配置合并）

        Returns:
            bool: 更新成功返回True，如果名称不存在返回False
        """
        all_configs = self._load_config()

        if name not in all_configs:
            return False

        # 合并配置
        existing_config = all_configs[name]
        existing_config.update(config)
        existing_config["name"] = name  # 确保名称不被覆盖
        existing_config["updated_at"] = datetime.now().isoformat()

        all_configs[name] = existing_config
        self._save_config(all_configs)
        return True

    def delete(self, name: str) -> bool:
        """
        删除RAG服务配置

        Args:
            name: RAG服务名称

        Returns:
            bool: 删除成功返回True，如果名称不存在返回False
        """
        all_configs = self._load_config()

        if name not in all_configs:
            return False

        del all_configs[name]
        self._save_config(all_configs)
        return True

    def list_names(self) -> List[str]:
        """
        获取所有RAG服务名称列表

        Returns:
            List[str]: 服务名称列表
        """
        all_configs = self._load_config()
        return list(all_configs.keys())

    def exists(self, name: str) -> bool:
        """
        检查RAG服务是否存在

        Args:
            name: RAG服务名称

        Returns:
            bool: 存在返回True，否则返回False
        """
        all_configs = self._load_config()
        return name in all_configs

    def get_by_port(self, port: int) -> Optional[Dict[str, Any]]:
        """
        根据端口号查找RAG服务配置

        Args:
            port: 端口号

        Returns:
            Dict: 配置字典，如果未找到返回None
        """
        all_configs = self._load_config()

        for name, config in all_configs.items():
            if config.get("port") == port:
                return config

        return None

    def get_running_services(self) -> Dict[str, Any]:
        """
        获取所有运行中的RAG服务

        Returns:
            Dict: 运行中的服务配置字典
        """
        all_configs = self._load_config()
        running_services = {}

        for name, config in all_configs.items():
            if config.get("status") == "running":
                running_services[name] = config

        return running_services

    def update_status(self, name: str, status: str, **kwargs) -> bool:
        """
        更新RAG服务状态

        Args:
            name: RAG服务名称
            status: 新状态 ('running', 'stopped', 'error', etc.)
            **kwargs: 其他要更新的状态信息（如process_id, stdout_fd等）

        Returns:
            bool: 更新成功返回True，如果名称不存在返回False
        """
        update_data = {"status": status, **kwargs}
        return self.update(name, update_data)

    def cleanup_stopped_services(self):
        """
        清理已停止服务的临时状态信息（如process_id, stdout_fd等）
        """
        all_configs = self._load_config()
        modified = False

        for name, config in all_configs.items():
            if config.get("status") == "stopped":
                # 清理临时状态字段
                temp_fields = [
                    "process_id",
                    "stdout_fd",
                    "stderr_fd",
                    "cache_build_task_id",
                ]
                for field in temp_fields:
                    if field in config:
                        del config[field]
                        modified = True

        if modified:
            self._save_config(all_configs)


# 创建全局实例
rag_manager = RAGConfigManager()


# 便捷函数
def create_rag_config(name: str, config: Dict[str, Any]) -> bool:
    """创建RAG配置的便捷函数"""
    return rag_manager.create(name, config)


def get_rag_config(name: Optional[str] = None) -> Optional[Dict[str, Any]]:
    """获取RAG配置的便捷函数"""
    return rag_manager.read(name)


def update_rag_config(name: str, config: Dict[str, Any]) -> bool:
    """更新RAG配置的便捷函数"""
    return rag_manager.update(name, config)


def delete_rag_config(name: str) -> bool:
    """删除RAG配置的便捷函数"""
    return rag_manager.delete(name)


def list_rag_names() -> List[str]:
    """获取所有RAG服务名称的便捷函数"""
    return rag_manager.list_names()


def rag_exists(name: str) -> bool:
    """检查RAG服务是否存在的便捷函数"""
    return rag_manager.exists(name)


def get_rag_by_port(port: int) -> Optional[Dict[str, Any]]:
    """根据端口获取RAG配置的便捷函数"""
    return rag_manager.get_by_port(port)


def get_running_rags() -> Dict[str, Any]:
    """获取运行中RAG服务的便捷函数"""
    return rag_manager.get_running_services()


def update_rag_status(name: str, status: str, **kwargs) -> bool:
    """更新RAG服务状态的便捷函数"""
    return rag_manager.update_status(name, status, **kwargs)



from autocoder.common import (AutoCoderArgs,ExecuteSteps,
                              ExecuteStep,EnvInfo,
                              detect_env,chat_with_llm_step_by_step,SourceCode)
from autocoder.common.JupyterClient import JupyterNotebook
from autocoder.common.ShellClient import ShellClient
from autocoder.common.types import Mode,StepNum
from typing import Optional,Dict,Any,List
import byzerllm
from loguru import logger


class CodeAutoExecute:
    def __init__(self,llm,args,mode:Mode=Mode.SINGLE_ROUND) -> None:
        self.llm = llm
        self.args = args
        self.mode = mode

    @byzerllm.prompt(llm=lambda self: self.llm)
    def get_execute_steps(self,query:str,                          
                          context:Optional[str]=None,
                          source_code:Optional[str]=None,mode:Mode=Mode.SINGLE_ROUND)->ExecuteSteps:
        '''        
        根据用户的问题，对问题进行拆解，然后生成执行步骤。

        环境信息如下:
        操作系统: {{ env_info.os_name }} {{ env_info.os_version }}  
        Python版本: {{ env_info.python_version }}
        {%- if env_info.conda_env %}
        Conda环境: {{ env_info.conda_env }}
        {%- endif %}
        {%- if env_info.virtualenv %}  
        虚拟环境: {{ env_info.virtualenv }}
        {%- endif %}
        {%- if env_info.has_bash %} 
        支持Bash
        {%- else %}
        不支持Bash
        {%- endif %}

        {%- if source_code %}
        下面是一系列文件以及它们的源码：
        {{ source_code }}
        {%- endif %}

        {%- if context %}
        现在请参考下面内容：

        {{ context }}
        {%- endif %}

        用户的问题是：{{ query }} 

        {{ mode }}       
        ''' 
        parameters= {
            "env_info":detect_env()
        }
        if mode == Mode.MULTI_ROUND:
            parameters["mode"] = "每次生成一个执行步骤，然后询问我是否继续，当我回复继续，继续生成下一个执行步骤。"
        return parameters    

    @byzerllm.prompt(llm=lambda self: self.llm)
    def get_step_num(self,query:str,                          
                          context:Optional[str]=None,
                          source_code:Optional[str]=None)->StepNum:
        '''        
        根据用户的问题，对问题进行拆解，然后生成执行步骤。

        环境信息如下:
        操作系统: {{ env_info.os_name }} {{ env_info.os_version }}  
        Python版本: {{ env_info.python_version }}
        {%- if env_info.conda_env %}
        Conda环境: {{ env_info.conda_env }}
        {%- endif %}
        {%- if env_info.virtualenv %}  
        虚拟环境: {{ env_info.virtualenv }}
        {%- endif %}
        {%- if env_info.has_bash %} 
        支持Bash
        {%- else %}
        不支持Bash
        {%- endif %}

        {%- if source_code %}
        下面是一系列文件以及它们的源码：
        {{ source_code }}
        {%- endif %}

        现在请参考下面内容：

        {{ context }}        

        用户的问题是：{{ query }}  

        根据问题，回答以下内容：
        1. 详细的执行步骤，每个步骤需要包含一个shell/python 代码块。
        2. 总共有多少步      
        '''  
        return {
            "env_info":detect_env()
        }  
    
    def execute_steps(self, steps: ExecuteSteps) -> str:
        jupyter_client = JupyterNotebook()
        shell_client = ShellClient(working_dir=steps.steps[0].cwd)
        
        try:
            output = ""
            for step in steps.steps:
                if step.lang and step.lang.lower() in ["python","py","python3"]:
                    output += f"Python Code:\n{step.code}\n"
                    output += "Output:\n"
                    result, error = jupyter_client.add_and_run(step.code)
                    output += result + "\n"
                    if error:
                        output += f"Error: {str(error)}\n"
                elif step.lang and step.lang.lower() in ["shell", "bash", "sh", "zsh", "ksh", "csh","powershell","cmd"]:  
                    output += f"Shell Command:\n{step.code}\n"
                    output += "Output:\n"                    
                    stdout, stderr = shell_client.add_and_run(step.code)
                    output += stdout + "\n"
                    if stderr:
                        output += f"Error: {stderr}\n"
                else:
                    output += f"Unknown step type: {step.lang}\n"
                
                output += "-" * 20 + "\n"
        except Exception as e:
            output += f"Error: {str(e)}\n"
        finally:    
            jupyter_client.close()
            shell_client.close()
        
        return output
    
    def run(self,query:str,context:str,source_code:str):
        
        if self.mode == Mode.SINGLE_ROUND:
            steps = self.get_execute_steps.run(query=query,context=context,source_code=source_code)

            print("\n\n=================================Execute Steps===========================================")
            print(steps)
            output = self.execute_steps(steps)

            print("\n\n=================================Execute Result===========================================")
            print(output)
            return
        if self.mode == Mode.MULTI_ROUND:
            step_num = self.get_step_num.run(query=query,context=context,source_code=source_code)
            
            final_v = ExecuteSteps(steps=[])            
            conversations = [{
                    "role":"user",
                    "content":step_num.content
                }] 
            print(f"=============================Collect AUTO STEPS===========================================",flush=True)
            print(f"{conversations[0]['role']}: {conversations[0]['content']}\n",flush=True)   

            (result,_) = chat_with_llm_step_by_step(self.llm,conversations=conversations,
                                                                response_class=ExecuteStep,
                                                                max_steps=step_num.step_num,                                                            
                                                                anti_quota_limit=self.args.anti_quota_limit)

            for item in result:            
                final_v.steps.append(item)                            
            
            # 执行步骤并保存结果
            if not final_v.steps:
                logger.error("No steps to execute, this may be caused by the model's response")
                return
            
            result = self.execute_steps(final_v)
            logger.info(result)            





    
        




import os
import re
import difflib
import traceback
from typing import Dict, Any, Optional, List, Tuple
import typing
from autocoder.agent.base_agentic.tools.base_tool_resolver import BaseToolResolver
from autocoder.agent.base_agentic.types import ReplaceInFileTool, ToolResult  # Import ToolResult from types
from autocoder.common.printer import Printer
from autocoder.common import AutoCoderArgs
from loguru import logger
from autocoder.common.auto_coder_lang import get_message_with_format
from autocoder.common.file_checkpoint.models import FileChange as CheckpointFileChange
from autocoder.common.file_checkpoint.manager import FileChangeManager as CheckpointFileChangeManager
from autocoder.linters.models import IssueSeverity, FileLintResult
if typing.TYPE_CHECKING:
    from ..base_agent import BaseAgent

class ReplaceInFileToolResolver(BaseToolResolver):
    def __init__(self, agent: Optional['BaseAgent'], tool: ReplaceInFileTool, args: AutoCoderArgs):
        super().__init__(agent, tool, args)
        self.tool: ReplaceInFileTool = tool  # For type hinting
        self.args = args
        self.shadow_manager = self.agent.shadow_manager if self.agent else None
        self.shadow_linter = self.agent.shadow_linter if self.agent else None

    def parse_diff(self, diff_content: str) -> List[Tuple[str, str]]:
        """
        Parses the diff content into a list of (search_block, replace_block) tuples.
        """
        blocks = []
        lines = diff_content.splitlines(keepends=True)
        i = 0
        n = len(lines)

        while i < n:
            line = lines[i]
            if line.strip() == "<<<<<<< SEARCH":
                i += 1
                search_lines = []
                # Accumulate search block
                while i < n and lines[i].strip() != "=======":
                    search_lines.append(lines[i])
                    i += 1
                if i >= n:
                    logger.warning("Unterminated SEARCH block found in diff content.")
                    break
                i += 1  # skip '======='
                replace_lines = []
                # Accumulate replace block
                while i < n and lines[i].strip() != ">>>>>>> REPLACE":
                    replace_lines.append(lines[i])
                    i += 1
                if i >= n:
                    logger.warning("Unterminated REPLACE block found in diff content.")
                    break
                i += 1  # skip '>>>>>>> REPLACE'

                search_block = ''.join(search_lines)
                replace_block = ''.join(replace_lines)
                blocks.append((search_block, replace_block))
            else:
                i += 1

        if not blocks and diff_content.strip():
            logger.warning(f"Could not parse any SEARCH/REPLACE blocks from diff: {diff_content}")
        return blocks

    def _filter_lint_issues(self, lint_result:FileLintResult, levels: List[IssueSeverity] = [IssueSeverity.ERROR, IssueSeverity.WARNING]):
        """
        过滤 lint 结果，只保留指定级别的问题
        
        参数:
            lint_result: 单个文件的 lint 结果对象
            levels: 要保留的问题级别列表，默认保留 ERROR 和 WARNING 级别
            
        返回:
            过滤后的 lint 结果对象（原对象的副本）
        """
        if not lint_result or not lint_result.issues:
            return lint_result
            
        # 创建一个新的 issues 列表，只包含指定级别的问题
        filtered_issues = []
        for issue in lint_result.issues:
            if issue.severity in levels:                
                filtered_issues.append(issue)
                
        # 更新 lint_result 的副本
        filtered_result = lint_result
        filtered_result.issues = filtered_issues
        
        # 更新计数
        filtered_result.error_count = sum(1 for issue in filtered_result.issues if issue.severity == IssueSeverity.ERROR)
        filtered_result.warning_count = sum(1 for issue in filtered_result.issues if issue.severity == IssueSeverity.WARNING)
        filtered_result.info_count = sum(1 for issue in filtered_result.issues if issue.severity == IssueSeverity.INFO)
        
        return filtered_result
        
    def _format_lint_issues(self, lint_result:FileLintResult):
        """
        将 lint 结果格式化为可读的文本格式
        
        参数:
            lint_result: 单个文件的 lint 结果对象
            
        返回:
            str: 格式化的问题描述
        """
        formatted_issues = []
        
        for issue in lint_result.issues:
            severity = "错误" if issue.severity.value == 3 else "警告" if issue.severity.value == 2 else "信息"
            line_info = f"第{issue.position.line}行"
            if issue.position.column:
                line_info += f", 第{issue.position.column}列"
            
            formatted_issues.append(
                f"  - [{severity}] {line_info}: {issue.message} (规则: {issue.code})"
            )
        
        return "\n".join(formatted_issues)

    

    def replace_in_file_normal(self, file_path: str, diff_content: str, source_dir: str, abs_project_dir: str, abs_file_path: str) -> ToolResult:
        """Replace content in file directly without using shadow manager"""
        try:
            # Read original content
            if not os.path.exists(abs_file_path):
                return ToolResult(success=False, message=get_message_with_format("replace_in_file.file_not_found", file_path=file_path))
            if not os.path.isfile(abs_file_path):
                return ToolResult(success=False, message=get_message_with_format("replace_in_file.not_a_file", file_path=file_path))

            with open(abs_file_path, 'r', encoding='utf-8', errors='replace') as f:
                original_content = f.read()

            parsed_blocks = self.parse_diff(diff_content)
            if not parsed_blocks:
                return ToolResult(success=False, message=get_message_with_format("replace_in_file.no_valid_blocks"))

            current_content = original_content
            applied_count = 0
            errors = []

            # Apply blocks sequentially
            for i, (search_block, replace_block) in enumerate(parsed_blocks):
                start_index = current_content.find(search_block)

                if start_index != -1:
                    current_content = current_content[:start_index] + replace_block + current_content[start_index + len(search_block):]
                    applied_count += 1
                    logger.info(f"Applied SEARCH/REPLACE block {i+1} in file {file_path}")
                else:
                    error_message = f"SEARCH block {i+1} not found in the current file content. Content to search:\n---\n{search_block}\n---"
                    logger.warning(error_message)
                    context_start = max(0, original_content.find(search_block[:20]) - 100)
                    context_end = min(len(original_content), context_start + 200 + len(search_block[:20]))
                    logger.warning(f"Approximate context in file:\n---\n{original_content[context_start:context_end]}\n---")
                    errors.append(error_message)

            if applied_count == 0 and errors:
                return ToolResult(success=False, message=get_message_with_format("replace_in_file.apply_failed", errors="\n".join(errors)))

            # Write the modified content back to file
            if self.agent and self.agent.checkpoint_manager:
                changes = {
                    file_path: CheckpointFileChange(
                        file_path=file_path,
                        content=current_content,
                        is_deletion=False,
                        is_new=True
                    )
                }
                change_group_id = self.args.event_file
                
                self.agent.checkpoint_manager.apply_changes_with_conversation(
                            changes=changes,
                            conversations=self.agent.current_conversations,
                            change_group_id=change_group_id,
                            metadata={"event_file": self.args.event_file}
                        )                                
            else:
                with open(abs_file_path, 'w', encoding='utf-8') as f:
                    f.write(current_content)
            
            logger.info(f"Successfully applied {applied_count}/{len(parsed_blocks)} changes to file: {file_path}")

            # 新增：执行代码质量检查
            lint_results = None
            lint_message = ""
            formatted_issues = ""
            has_lint_issues = False
            
            # 检查是否启用了Lint功能
            enable_lint = self.args.enable_auto_fix_lint
            logger.info(f"检查Lint功能状态: enable_lint={enable_lint}")
            
            if enable_lint:
                try:
                    if self.agent.linter:
                        lint_results = self.agent.linter.lint_file(file_path)
                        if lint_results and lint_results.issues:
                            # 过滤 lint 结果，只保留 ERROR 和 WARNING 级别的问题
                            filtered_results = self._filter_lint_issues(lint_results)
                            if filtered_results.issues:
                                has_lint_issues = True
                                # 格式化 lint 问题
                                formatted_issues = self._format_lint_issues(filtered_results)
                                lint_message = f"\n\n代码质量检查发现 {len(filtered_results.issues)} 个问题"
                except Exception as e:
                    logger.error(f"Lint 检查失败: {str(e)}")
                    lint_message = "\n\n尝试进行代码质量检查时出错。"
            else:
                logger.info("代码质量检查已禁用")

            # 构建包含 lint 结果的返回消息
            if errors:
                message = get_message_with_format("replace_in_file.apply_success_with_warnings", 
                                                  applied=applied_count, 
                                                  total=len(parsed_blocks), 
                                                  file_path=file_path,
                                                  errors="\n".join(errors))
            else:
                message = get_message_with_format("replace_in_file.apply_success", 
                                                  applied=applied_count, 
                                                  total=len(parsed_blocks), 
                                                  file_path=file_path)                        

            # 变更跟踪，回调Agent
            if self.agent:
                rel_path = os.path.relpath(abs_file_path, abs_project_dir)
                self.agent.record_file_change(rel_path, "modified", diff=diff_content, content=current_content)
                        
            # 附加 lint 结果到返回内容
            result_content = {
                "content": current_content,
            }
            
            # 只有在启用Lint时才添加Lint结果
            if enable_lint:
                message = message + "\n" + lint_message
                result_content["lint_results"] = {
                    "has_issues": has_lint_issues,
                    "issues": formatted_issues if has_lint_issues else None
                }

            return ToolResult(success=True, message=message, content=result_content)
        except Exception as e:
            logger.error(f"Error writing replaced content to file '{file_path}': {str(e)}")
            return ToolResult(success=False, message=get_message_with_format("replace_in_file.write_error", error=str(e)))

    def resolve(self) -> ToolResult:
        """Resolve the replace in file tool by calling the appropriate implementation"""
        file_path = self.tool.path
        diff_content = self.tool.diff
        source_dir = self.args.source_dir or "."
        abs_project_dir = os.path.abspath(source_dir)
        abs_file_path = os.path.abspath(os.path.join(source_dir, file_path))

        # Security check
        if not abs_file_path.startswith(abs_project_dir):
            return ToolResult(success=False, message=get_message_with_format("replace_in_file.access_denied", file_path=file_path))
                
        return self.replace_in_file_normal(file_path, diff_content, source_dir, abs_project_dir, abs_file_path)



"""系统初始化模块"""

import subprocess
from prompt_toolkit import prompt
from prompt_toolkit.shortcuts import radiolist_dialog
from prompt_toolkit.formatted_text import HTML
from autocoder.chat_auto_coder_lang import get_message


def initialize_system(args):
    """初始化系统环境（Ray、模型等）"""
    if args.product_mode == "lite":
        return

    print(f"\n\033[1;34m{get_message('initializing')}\033[0m")

    def print_status(message, status):
        if status == "success":
            print(f"\033[32m✓ {message}\033[0m")
        elif status == "warning":
            print(f"\033[33m! {message}\033[0m")
        elif status == "error":
            print(f"\033[31m✗ {message}\033[0m")
        else:
            print(f"  {message}")

    # Check if Ray is running
    print_status(get_message("checking_ray"), "")
    ray_status = subprocess.run(["ray", "status"], capture_output=True, text=True)
    if ray_status.returncode != 0:
        print_status(get_message("ray_not_running"), "warning")
        try:
            subprocess.run(["ray", "start", "--head"], check=True)
            print_status(get_message("ray_start_success"), "success")
        except subprocess.CalledProcessError:
            print_status(get_message("ray_start_fail"), "error")
            return
    else:
        print_status(get_message("ray_running"), "success")

    # Check if deepseek_chat model is available
    print_status(get_message("checking_model"), "")
    try:
        result = subprocess.run(
            ["easy-byzerllm", "chat", "v3_chat", "你好"],
            capture_output=True,
            text=True,
            timeout=30,
        )
        if result.returncode == 0:
            print_status(get_message("model_available"), "success")
            print_status(get_message("init_complete_final"), "success")
            return
    except subprocess.TimeoutExpired:
        print_status(get_message("model_timeout"), "error")
    except subprocess.CalledProcessError:
        print_status(get_message("model_error"), "error")

    # If deepseek_chat is not available, prompt user to choose a provider
    print_status(get_message("model_not_available"), "warning")
    choice = radiolist_dialog(
        title=get_message("provider_selection"),
        text=get_message("provider_selection"),
        values=[
            ("1", "Deepseek官方(https://www.deepseek.com/)"),
        ],
    ).run()

    if choice is None:
        print_status(get_message("no_provider"), "error")
        return

    api_key = prompt(HTML(f"<b>{get_message('enter_api_key')} </b>"))

    if choice == "1":
        print_status(get_message("deploying_model").format("Deepseek官方"), "")

        deploy_cmd = [
            "byzerllm",
            "deploy",
            "--pretrained_model_type",
            "saas/openai",
            "--cpus_per_worker",
            "0.001",
            "--gpus_per_worker",
            "0",
            "--worker_concurrency",
            "1000",
            "--num_workers",
            "1",
            "--infer_params",
            f"saas.base_url=https://api.deepseek.com/v1 saas.api_key={api_key} saas.model=deepseek-chat",
            "--model",
            "v3_chat",
        ]

    try:
        subprocess.run(deploy_cmd, check=True)
        print_status(get_message("deploy_complete"), "success")
    except subprocess.CalledProcessError:
        print_status(get_message("deploy_fail"), "error")
        return

    # Validate the deployment
    print_status(get_message("validating_deploy"), "")
    try:
        validation_result = subprocess.run(
            ["easy-byzerllm", "chat", "v3_chat", "你好"],
            capture_output=True,
            text=True,
            timeout=30,
            check=True,
        )
        print_status(get_message("validation_success"), "success")
    except (subprocess.TimeoutExpired, subprocess.CalledProcessError):
        print_status(get_message("validation_fail"), "error")
        print_status(get_message("manual_start"), "warning")
        print_status("easy-byzerllm chat v3_chat 你好", "")

    print_status(get_message("init_complete_final"), "success")



import tabulate
from autocoder.db.store import TokenCounter
from typing import List
def print_table(token_counters:List[TokenCounter]):    
    headers =  TokenCounter.model_fields.keys()
    table_data = [[getattr(counter, name) for name in headers] for counter in token_counters]    
    table_output = tabulate.tabulate(table_data, headers, tablefmt="grid")    
    print(table_output,flush=True)    


"""
Async command messages for internationalization
Contains all messages used by the async command functionality
"""

ASYNC_COMMAND_MESSAGES = {
    # 错误消息
    "async_task_param_error": {
        "en": "Parameter Error",
        "zh": "参数错误",
        "ja": "パラメータエラー",
        "ar": "خطأ في المعامل",
        "ru": "Ошибка параметра",
    },
    "async_provide_task_id": {
        "en": "Please provide the task ID to terminate",
        "zh": "请提供要终止的任务ID",
        "ja": "終了するタスクIDを提供してください",
        "ar": "يرجى تقديم معرف المهمة للإنهاء",
        "ru": "Пожалуйста, укажите ID задачи для завершения",
    },
    "async_name_required": {
        "en": "The /name parameter is required for async tasks.\n\nUsage: /async /name <task_name> <your_query>\n\nExample:\n  /async /name my_task /model gpt-4 implement user authentication",
        "zh": "异步任务必须提供 /name 参数。\n\n用法: /async /name <任务名称> <你的查询>\n\n示例:\n  /async /name my_task /model gpt-4 实现用户认证功能",
        "ja": "非同期タスクには /name パラメータが必要です。\n\n使用法: /async /name <タスク名> <クエリ>\n\n例:\n  /async /name my_task /model gpt-4 ユーザー認証を実装する",
        "ar": "معامل /name مطلوب للمهام غير المتزامنة.\n\nالاستخدام: /async /name <اسم_المهمة> <استعلامك>\n\nمثال:\n  /async /name my_task /model gpt-4 تنفيذ مصادقة المستخدم",
        "ru": "Параметр /name обязателен для асинхронных задач.\n\nИспользование: /async /name <имя_задачи> <ваш_запрос>\n\nПример:\n  /async /name my_task /model gpt-4 реализовать аутентификацию пользователя",
    },
    "async_task_not_found": {
        "en": "Task ID not found: {{task_id}}",
        "zh": "未找到任务ID: {{task_id}}",
        "ja": "タスクIDが見つかりません: {{task_id}}",
        "ar": "لم يتم العثور على معرف المهمة: {{task_id}}",
        "ru": "ID задачи не найден: {{task_id}}",
    },
    "async_task_not_exist": {
        "en": "Task Not Exist",
        "zh": "任务不存在",
        "ja": "タスクが存在しません",
        "ar": "المهمة غير موجودة",
        "ru": "Задача не существует",
    },
    "async_task_status_error": {
        "en": "Task Status Error",
        "zh": "任务状态错误",
        "ja": "タスクステータスエラー",
        "ar": "خطأ في حالة المهمة",
        "ru": "Ошибка статуса задачи",
    },
    "async_task_cannot_terminate": {
        "en": "Task {{task_id}} is currently in {{status}} status and cannot be terminated",
        "zh": "任务 {{task_id}} 当前状态为 {{status}}，无法终止",
        "ja": "タスク {{task_id}} は現在 {{status}} ステータスで、終了できません",
        "ar": "المهمة {{task_id}} في حالة {{status}} حالياً ولا يمكن إنهاؤها",
        "ru": "Задача {{task_id}} в настоящее время имеет статус {{status}} и не может быть завершена",
    },
    "async_missing_psutil": {
        "en": "Missing psutil dependency, cannot terminate process",
        "zh": "缺少 psutil 依赖，无法终止进程",
        "ja": "psutil の依存関係が見つからないため、プロセスを終了できません",
        "ar": "مكتبة psutil مفقودة، لا يمكن إنهاء العملية",
        "ru": "Отсутствует зависимость psutil, невозможно завершить процесс",
    },
    "async_dependency_missing": {
        "en": "Dependency Missing",
        "zh": "依赖缺失",
        "ja": "依存関係の不足",
        "ar": "التبعية مفقودة",
        "ru": "Отсутствует зависимость",
    },
    "async_process_not_exist": {
        "en": "Process {{pid}} does not exist, may have already finished",
        "zh": "进程 {{pid}} 不存在，可能已经结束",
        "ja": "プロセス {{pid}} は存在しません。既に終了している可能性があります",
        "ar": "العملية {{pid}} غير موجودة، ربما انتهت بالفعل",
        "ru": "Процесс {{pid}} не существует, возможно, уже завершён",
    },
    "async_process_not_exist_title": {
        "en": "Process Not Exist",
        "zh": "进程不存在",
        "ja": "プロセスが存在しません",
        "ar": "العملية غير موجودة",
        "ru": "Процесс не существует",
    },
    "async_terminate_process_error": {
        "en": "Error occurred while terminating process: {{error}}",
        "zh": "终止进程时发生错误: {{error}}",
        "ja": "プロセスの終了中にエラーが発生しました: {{error}}",
        "ar": "حدث خطأ أثناء إنهاء العملية: {{error}}",
        "ru": "Ошибка при завершении процесса: {{error}}",
    },
    "async_terminate_failed": {
        "en": "Terminate Failed",
        "zh": "终止失败",
        "ja": "終了失敗",
        "ar": "فشل الإنهاء",
        "ru": "Завершение не выполнено",
    },
    "async_no_valid_pid": {
        "en": "Task {{task_id}} has no valid PID information",
        "zh": "任务 {{task_id}} 没有有效的PID信息",
        "ja": "タスク {{task_id}} には有効なPID情報がありません",
        "ar": "المهمة {{task_id}} لا تحتوي على معلومات PID صالحة",
        "ru": "Задача {{task_id}} не содержит действительной информации о PID",
    },
    "async_pid_missing": {
        "en": "PID Information Missing",
        "zh": "PID信息缺失",
        "ja": "PID情報の不足",
        "ar": "معلومات PID مفقودة",
        "ru": "Отсутствует информация о PID",
    },
    "async_kill_command_error": {
        "en": "Error occurred while processing kill command: {{error}}",
        "zh": "处理kill命令时发生错误: {{error}}",
        "ja": "killコマンドの処理中にエラーが発生しました: {{error}}",
        "ar": "حدث خطأ أثناء معالجة أمر القتل: {{error}}",
        "ru": "Ошибка при обработке команды завершения: {{error}}",
    },
    "async_processing_error": {
        "en": "Processing Error",
        "zh": "处理错误",
        "ja": "処理エラー",
        "ar": "خطأ في المعالجة",
        "ru": "Ошибка обработки",
    },
    "async_task_detail_load_error": {
        "en": "Error occurred while loading task details: {{error}}",
        "zh": "加载任务详情时发生错误: {{error}}",
        "ja": "タスクの詳細の読み込み中にエラーが発生しました: {{error}}",
        "ar": "حدث خطأ أثناء تحميل تفاصيل المهمة: {{error}}",
        "ru": "Ошибка при загрузке деталей задачи: {{error}}",
    },
    "async_task_list_error": {
        "en": "Error occurred while getting task list: {{error}}",
        "zh": "获取任务列表时发生错误: {{error}}",
        "ja": "タスクリストの取得中にエラーが発生しました: {{error}}",
        "ar": "حدث خطأ أثناء الحصول على قائمة المهام: {{error}}",
        "ru": "Ошибка при получении списка задач: {{error}}",
    },
    # 成功消息
    "async_task_terminated_success": {
        "en": "Task {{task_id}} (PID: {{pid}}) has been successfully terminated",
        "zh": "任务 {{task_id}} (PID: {{pid}}) 已成功终止",
        "ja": "タスク {{task_id}} (PID: {{pid}}) が正常に終了されました",
        "ar": "تم إنهاء المهمة {{task_id}} (PID: {{pid}}) بنجاح",
        "ru": "Задача {{task_id}} (PID: {{pid}}) успешно завершена",
    },
    "async_terminate_success": {
        "en": "Task Terminate Success",
        "zh": "任务终止成功",
        "ja": "タスク終了成功",
        "ar": "نجح إنهاء المهمة",
        "ru": "Задача успешно завершена",
    },
    "async_task_status_updated": {
        "en": "Task status updated",
        "zh": "任务状态已更新",
        "ja": "タスクステータスが更新されました",
        "ar": "تم تحديث حالة المهمة",
        "ru": "Статус задачи обновлён",
    },
    # 任务状态
    "async_task_status_running": {
        "en": "[blue]Running[/blue]",
        "zh": "[blue]运行中[/blue]",
        "ja": "[blue]実行中[/blue]",
        "ar": "[blue]قيد التشغيل[/blue]",
        "ru": "[blue]Выполняется[/blue]",
    },
    "async_task_status_completed": {
        "en": "[green]Completed[/green]",
        "zh": "[green]已完成[/green]",
        "ja": "[green]完了[/green]",
        "ar": "[green]مكتمل[/green]",
        "ru": "[green]Завершено[/green]",
    },
    "async_task_status_failed": {
        "en": "[red]Failed[/red]",
        "zh": "[red]失败[/red]",
        "ja": "[red]失敗[/red]",
        "ar": "[red]فاشل[/red]",
        "ru": "[red]Неудачно[/red]",
    },
    # 任务列表
    "async_task_list_title": {
        "en": "Async Task List",
        "zh": "异步任务列表",
        "ja": "非同期タスクリスト",
        "ar": "قائمة المهام غير المتزامنة",
        "ru": "Список асинхронных задач",
    },
    "async_task_list_no_tasks": {
        "en": "No tasks found",
        "zh": "没有找到任务",
        "ja": "タスクが見つかりません",
        "ar": "لم يتم العثور على مهام",
        "ru": "Задачи не найдены",
    },
    "async_task_table_id": {
        "en": "Task ID",
        "zh": "任务ID",
        "ja": "タスクID",
        "ar": "معرف المهمة",
        "ru": "ID задачи",
    },
    "async_task_table_status": {
        "en": "Status",
        "zh": "状态",
        "ja": "ステータス",
        "ar": "الحالة",
        "ru": "Статус",
    },
    "async_task_table_model": {
        "en": "Model",
        "zh": "模型",
        "ja": "モデル",
        "ar": "النموذج",
        "ru": "Модель",
    },
    "async_task_table_created": {
        "en": "Created",
        "zh": "创建时间",
        "ja": "作成時刻",
        "ar": "وقت الإنشاء",
        "ru": "Создано",
    },
    "async_task_table_query": {
        "en": "Query",
        "zh": "查询",
        "ja": "クエリ",
        "ar": "الاستعلام",
        "ru": "Запрос",
    },
    "async_task_table_log": {
        "en": "Log File",
        "zh": "日志文件",
        "ja": "ログファイル",
        "ar": "ملف السجل",
        "ru": "Файл журнала",
    },
    "async_task_list_summary": {
        "en": "Total: {{total}} | Completed: {{completed}} | Running: {{running}} | Failed: {{failed}}",
        "zh": "总计: {{total}} | 已完成: {{completed}} | 运行中: {{running}} | 失败: {{failed}}",
        "ja": "合計: {{total}} | 完了: {{completed}} | 実行中: {{running}} | 失敗: {{failed}}",
        "ar": "المجموع: {{total}} | مكتمل: {{completed}} | قيد التشغيل: {{running}} | فاشل: {{failed}}",
        "ru": "Всего: {{total}} | Завершено: {{completed}} | Выполняется: {{running}} | Неудачно: {{failed}}",
    },
    # 任务详情
    "async_task_detail_title": {
        "en": "Task Details",
        "zh": "任务详情",
        "ja": "タスクの詳細",
        "ar": "تفاصيل المهمة",
        "ru": "Детали задачи",
    },
    "async_task_detail_not_found": {
        "en": "Task ID {{task_id}} not found",
        "zh": "未找到任务ID {{task_id}}",
        "ja": "タスクID {{task_id}} が見つかりません",
        "ar": "لم يتم العثور على معرف المهمة {{task_id}}",
        "ru": "ID задачи {{task_id}} не найден",
    },
    "async_task_field_id": {
        "en": "Task ID",
        "zh": "任务ID",
        "ja": "タスクID",
        "ar": "معرف المهمة",
        "ru": "ID задачи",
    },
    "async_task_field_status": {
        "en": "Status",
        "zh": "状态",
        "ja": "ステータス",
        "ar": "الحالة",
        "ru": "Статус",
    },
    "async_task_field_model": {
        "en": "Model",
        "zh": "模型",
        "ja": "モデル",
        "ar": "النموذج",
        "ru": "Модель",
    },
    "async_task_field_split_mode": {
        "en": "Split Mode",
        "zh": "分割模式",
        "ja": "分割モード",
        "ar": "وضع التقسيم",
        "ru": "Режим разделения",
    },
    "async_task_field_bg_mode": {
        "en": "Background Mode",
        "zh": "后台模式",
        "ja": "バックグラウンドモード",
        "ar": "وضع الخلفية",
        "ru": "Фоновый режим",
    },
    "async_task_field_pr_mode": {
        "en": "Pull Request Mode",
        "zh": "拉取请求模式",
        "ja": "プルリクエストモード",
        "ar": "وضع طلب السحب",
        "ru": "Режим запроса на слияние",
    },
    "async_task_field_created": {
        "en": "Created At",
        "zh": "创建时间",
        "ja": "作成時刻",
        "ar": "وقت الإنشاء",
        "ru": "Создано в",
    },
    "async_task_field_completed": {
        "en": "Completed At",
        "zh": "完成时间",
        "ja": "完了時刻",
        "ar": "وقت الإكمال",
        "ru": "Завершено в",
    },
    "async_task_field_duration": {
        "en": "Duration",
        "zh": "耗时",
        "ja": "所要時間",
        "ar": "المدة",
        "ru": "Длительность",
    },
    "async_task_duration_format": {
        "en": "{{duration}} seconds",
        "zh": "{{duration}} 秒",
        "ja": "{{duration}} 秒",
        "ar": "{{duration}} ثانية",
        "ru": "{{duration}} секунд",
    },
    "async_task_field_worktree_path": {
        "en": "Worktree Path",
        "zh": "工作树路径",
        "ja": "ワークツリーパス",
        "ar": "مسار شجرة العمل",
        "ru": "Путь рабочего дерева",
    },
    "async_task_field_original_path": {
        "en": "Original Path",
        "zh": "原始路径",
        "ja": "元のパス",
        "ar": "المسار الأصلي",
        "ru": "Исходный путь",
    },
    "async_task_field_log_file": {
        "en": "Log File",
        "zh": "日志文件",
        "ja": "ログファイル",
        "ar": "ملف السجل",
        "ru": "Файл журнала",
    },
    "async_task_field_success": {
        "en": "Success",
        "zh": "成功",
        "ja": "成功",
        "ar": "نجح",
        "ru": "Успех",
    },
    "async_task_field_output_preview": {
        "en": "Output Preview",
        "zh": "输出预览",
        "ja": "出力プレビュー",
        "ar": "معاينة الإخراج",
        "ru": "Предварительный просмотр вывода",
    },
    "async_task_field_error_preview": {
        "en": "Error Preview",
        "zh": "错误预览",
        "ja": "エラープレビュー",
        "ar": "معاينة الخطأ",
        "ru": "Предварительный просмотр ошибки",
    },
    "async_task_value_yes": {
        "en": "Yes",
        "zh": "是",
        "ja": "はい",
        "ar": "نعم",
        "ru": "Да",
    },
    "async_task_value_no": {
        "en": "No",
        "zh": "否",
        "ja": "いいえ",
        "ar": "لا",
        "ru": "Нет",
    },
    # 面板标题
    "async_task_panel_query": {
        "en": "User Query",
        "zh": "用户查询",
        "ja": "ユーザークエリ",
        "ar": "استعلام المستخدم",
        "ru": "Пользовательский запрос",
    },
    "async_task_panel_paths": {
        "en": "Path Information",
        "zh": "路径信息",
        "ja": "パス情報",
        "ar": "معلومات المسار",
        "ru": "Информация о путях",
    },
    "async_task_panel_error": {
        "en": "Error Information",
        "zh": "错误信息",
        "ja": "エラー情報",
        "ar": "معلومات الخطأ",
        "ru": "Информация об ошибке",
    },
    "async_task_panel_execution": {
        "en": "Execution Result",
        "zh": "执行结果",
        "ja": "実行結果",
        "ar": "نتيجة التنفيذ",
        "ru": "Результат выполнения",
    },
    "async_task_operation_hints": {
        "en": "Available Operations",
        "zh": "可用操作",
        "ja": "利用可能な操作",
        "ar": "العمليات المتاحة",
        "ru": "Доступные операции",
    },
    # 操作提示
    "async_task_hint_view_log": {
        "en": "📄 View log: cat {{log_file}}",
        "zh": "📄 查看日志: cat {{log_file}}",
        "ja": "📄 ログ表示: cat {{log_file}}",
        "ar": "📄 عرض السجل: cat {{log_file}}",
        "ru": "📄 Просмотр журнала: cat {{log_file}}",
    },
    "async_task_hint_enter_worktree": {
        "en": "📁 Enter worktree: cd {{worktree_path}}",
        "zh": "📁 进入工作树: cd {{worktree_path}}",
        "ja": "📁 ワークツリーに入る: cd {{worktree_path}}",
        "ar": "📁 دخول شجرة العمل: cd {{worktree_path}}",
        "ru": "📁 Войти в рабочее дерево: cd {{worktree_path}}",
    },
    "async_task_hint_back_to_list": {
        "en": "📋 Back to list: /async /list",
        "zh": "📋 返回列表: /async /list",
        "ja": "📋 リストに戻る: /async /list",
        "ar": "📋 العودة إلى القائمة: /async /list",
        "ru": "📋 Вернуться к списку: /async /list",
    },
    # 异步任务启动消息
    "async_task_started_message": {
        "en": "[green]Async task has been started![/green]\n\nModel: [yellow]{{model}}[/yellow]\nQuery: [blue]{{query}}[/blue]\n\n[bold]Task details can be viewed at:[/bold]\n[cyan]{{agent_dir}}[/cyan]\n\n[dim]Tip: The task is running in the background, you can continue using other features[/dim]",
        "zh": "[green]异步任务已启动！[/green]\n\n模型: [yellow]{{model}}[/yellow]\n查询: [blue]{{query}}[/blue]\n\n[bold]任务详情请查看:[/bold]\n[cyan]{{agent_dir}}[/cyan]\n\n[dim]提示: 任务正在后台执行，您可以继续使用其他功能[/dim]",
        "ja": "[green]非同期タスクが開始されました！[/green]\n\nモデル: [yellow]{{model}}[/yellow]\nクエリ: [blue]{{query}}[/blue]\n\n[bold]タスクの詳細は以下で確認できます:[/bold]\n[cyan]{{agent_dir}}[/cyan]\n\n[dim]ヒント: タスクはバックグラウンドで実行中です。他の機能を引き続き使用できます[/dim]",
        "ar": "[green]تم بدء المهمة غير المتزامنة![/green]\n\nالنموذج: [yellow]{{model}}[/yellow]\nالاستعلام: [blue]{{query}}[/blue]\n\n[bold]يمكن عرض تفاصيل المهمة في:[/bold]\n[cyan]{{agent_dir}}[/cyan]\n\n[dim]نصيحة: المهمة تعمل في الخلفية، يمكنك الاستمرار في استخدام الميزات الأخرى[/dim]",
        "ru": "[green]Асинхронная задача запущена![/green]\n\nМодель: [yellow]{{model}}[/yellow]\nЗапрос: [blue]{{query}}[/blue]\n\n[bold]Детали задачи можно просмотреть в:[/bold]\n[cyan]{{agent_dir}}[/cyan]\n\n[dim]Совет: Задача выполняется в фоновом режиме, вы можете продолжать использовать другие функции[/dim]",
    },
    # 异步任务启动消息（带名称）
    "async_task_started_message_with_name": {
        "en": "[green]Async task has been started![/green]\n\nModel: [yellow]{{model}}[/yellow]\nQuery: [blue]{{query}}[/blue]\n\n[bold]Worktree Path:[/bold]\n[cyan]{{tasks_dir}}[/cyan]\n\n[bold]Meta Information:[/bold]\n[cyan]{{meta_file}}[/cyan]\n\n[bold]Task details can be viewed at:[/bold]\n[cyan]{{agent_dir}}[/cyan]\n\n[dim]Tip: The task is running in the background, you can continue using other features[/dim]",
        "zh": "[green]异步任务已启动！[/green]\n\n模型: [yellow]{{model}}[/yellow]\n查询: [blue]{{query}}[/blue]\n\n[bold]工作目录路径:[/bold]\n[cyan]{{tasks_dir}}[/cyan]\n\n[bold]元信息文件:[/bold]\n[cyan]{{meta_file}}[/cyan]\n\n[bold]任务详情请查看:[/bold]\n[cyan]{{agent_dir}}[/cyan]\n\n[dim]提示: 任务正在后台执行，您可以继续使用其他功能[/dim]",
        "ja": "[green]非同期タスクが開始されました！[/green]\n\nモデル: [yellow]{{model}}[/yellow]\nクエリ: [blue]{{query}}[/blue]\n\n[bold]ワークツリーパス:[/bold]\n[cyan]{{tasks_dir}}[/cyan]\n\n[bold]メタ情報:[/bold]\n[cyan]{{meta_file}}[/cyan]\n\n[bold]タスクの詳細は以下で確認できます:[/bold]\n[cyan]{{agent_dir}}[/cyan]\n\n[dim]ヒント: タスクはバックグラウンドで実行中です。他の機能を引き続き使用できます[/dim]",
        "ar": "[green]تم بدء المهمة غير المتزامنة![/green]\n\nالنموذج: [yellow]{{model}}[/yellow]\nالاستعلام: [blue]{{query}}[/blue]\n\n[bold]مسار شجرة العمل:[/bold]\n[cyan]{{tasks_dir}}[/cyan]\n\n[bold]معلومات التعريف:[/bold]\n[cyan]{{meta_file}}[/cyan]\n\n[bold]يمكن عرض تفاصيل المهمة في:[/bold]\n[cyan]{{agent_dir}}[/cyan]\n\n[dim]نصيحة: المهمة تعمل في الخلفية، يمكنك الاستمرار في استخدام الميزات الأخرى[/dim]",
        "ru": "[green]Асинхронная задача запущена![/green]\n\nМодель: [yellow]{{model}}[/yellow]\nЗапрос: [blue]{{query}}[/blue]\n\n[bold]Путь рабочего дерева:[/bold]\n[cyan]{{tasks_dir}}[/cyan]\n\n[bold]Мета информация:[/bold]\n[cyan]{{meta_file}}[/cyan]\n\n[bold]Детали задачи можно просмотреть в:[/bold]\n[cyan]{{agent_dir}}[/cyan]\n\n[dim]Совет: Задача выполняется в фоновом режиме, вы можете продолжать использовать другие функции[/dim]",
    },
    "async_task_title": {
        "en": "🚀 Async Task",
        "zh": "🚀 异步任务",
        "ja": "🚀 非同期タスク",
        "ar": "🚀 مهمة غير متزامنة",
        "ru": "🚀 Асинхронная задача",
    },
    # Drop command messages
    "async_task_drop_running_warning": {
        "en": "Task {{task_id}} is currently running. It will be terminated before deletion.",
        "zh": "任务 {{task_id}} 正在运行中。将在删除前终止该任务。",
        "ja": "タスク {{task_id}} は現在実行中です。削除前に終了されます。",
        "ar": "المهمة {{task_id}} قيد التشغيل حالياً. سيتم إنهاؤها قبل الحذف.",
        "ru": "Задача {{task_id}} в настоящее время выполняется. Она будет завершена перед удалением.",
    },
    "async_task_terminated_before_drop": {
        "en": "Task {{task_id}} processes terminated before deletion:\n{{processes}}",
        "zh": "任务 {{task_id}} 的进程在删除前已终止:\n{{processes}}",
        "ja": "タスク {{task_id}} のプロセスが削除前に終了しました:\n{{processes}}",
        "ar": "تم إنهاء عمليات المهمة {{task_id}} قبل الحذف:\n{{processes}}",
        "ru": "Процессы задачи {{task_id}} завершены перед удалением:\n{{processes}}",
    },
    "async_task_drop_success": {
        "en": "Task {{task_id}} and all related files have been successfully deleted:\n{{deleted_items}}",
        "zh": "任务 {{task_id}} 及所有相关文件已成功删除:\n{{deleted_items}}",
        "ja": "タスク {{task_id}} とすべての関連ファイルが正常に削除されました:\n{{deleted_items}}",
        "ar": "تم حذف المهمة {{task_id}} وجميع الملفات ذات الصلة بنجاح:\n{{deleted_items}}",
        "ru": "Задача {{task_id}} и все связанные файлы успешно удалены:\n{{deleted_items}}",
    },
    "async_task_drop_title": {
        "en": "🗑️ Task Deleted",
        "zh": "🗑️ 任务已删除",
        "ja": "🗑️ タスクが削除されました",
        "ar": "🗑️ تم حذف المهمة",
        "ru": "🗑️ Задача удалена",
    },
    "async_task_drop_no_files": {
        "en": "Task {{task_id}} metadata deleted, but no related files were found to clean up.",
        "zh": "任务 {{task_id}} 的元数据已删除，但没有找到需要清理的相关文件。",
        "ja": "タスク {{task_id}} のメタデータは削除されましたが、クリーンアップする関連ファイルは見つかりませんでした。",
        "ar": "تم حذف بيانات التعريف للمهمة {{task_id}}، ولكن لم يتم العثور على ملفات ذات صلة للتنظيف.",
        "ru": "Метаданные задачи {{task_id}} удалены, но не найдено связанных файлов для очистки.",
    },
    "async_drop_command_error": {
        "en": "Error occurred while processing drop command: {{error}}",
        "zh": "处理drop命令时发生错误: {{error}}",
        "ja": "dropコマンドの処理中にエラーが発生しました: {{error}}",
        "ar": "حدث خطأ أثناء معالجة أمر الحذف: {{error}}",
        "ru": "Ошибка при обработке команды удаления: {{error}}",
    },
    # 模型配置错误消息
    "async_model_config_error": {
        "en": "Model Configuration Error",
        "zh": "模型配置错误",
        "ja": "モデル設定エラー",
        "ar": "خطأ في تكوين النموذج",
        "ru": "Ошибка конфигурации модели",
    },
    "async_model_not_found": {
        "en": "Model '{{model}}' is not configured in the system.\n\nAvailable models can be listed using: /models\n\nTo add a new model, please configure it in models.json file or use model management commands.",
        "zh": "模型 '{{model}}' 未在系统中配置。\n\n可用模型列表可通过以下命令查看: /models\n\n要添加新模型，请在 models.json 文件中配置或使用模型管理命令。",
        "ja": "モデル '{{model}}' はシステムで設定されていません。\n\n利用可能なモデルの一覧は以下のコマンドで確認できます: /models\n\n新しいモデルを追加するには、models.json ファイルで設定するか、モデル管理コマンドを使用してください。",
        "ar": "النموذج '{{model}}' لم يتم تكوينه في النظام.\n\nقائمة النماذج المتاحة يمكن عرضها باستخدام: /models\n\nلإضافة نموذج جديد، يرجى تكوينه في ملف models.json أو استخدام أوامر إدارة النماذج.",
        "ru": "Модель '{{model}}' не настроена в системе.\n\nСписок доступных моделей можно просмотреть с помощью: /models\n\nЧтобы добавить новую модель, настройте ее в файле models.json или используйте команды управления моделями.",
    },
    "async_model_key_missing": {
        "en": "Model '{{model}}' is configured but API key is missing.\n\nPlease configure API key for this model using:\n  /models --update {{model}} --api-key YOUR_API_KEY\n\nOr manually add key file at: ~/.auto-coder/keys/{{model}}.key",
        "zh": "模型 '{{model}}' 已配置但缺少 API 密钥。\n\n请使用以下命令为此模型配置 API 密钥:\n  /models --update {{model}} --api-key YOUR_API_KEY\n\n或在以下位置手动添加密钥文件: ~/.auto-coder/keys/{{model}}.key",
        "ja": "モデル '{{model}}' は設定されていますが、APIキーがありません。\n\n以下のコマンドを使用してこのモデルのAPIキーを設定してください:\n  /models --update {{model}} --api-key YOUR_API_KEY\n\nまたは以下の場所に手動でキーファイルを追加してください: ~/.auto-coder/keys/{{model}}.key",
        "ar": "النموذج '{{model}}' تم تكوينه ولكن مفتاح API مفقود.\n\nيرجى تكوين مفتاح API لهذا النموذج باستخدام:\n  /models --update {{model}} --api-key YOUR_API_KEY\n\nأو أضف ملف المفتاح يدوياً في: ~/.auto-coder/keys/{{model}}.key",
        "ru": "Модель '{{model}}' настроена, но отсутствует ключ API.\n\nПожалуйста, настройте ключ API для этой модели с помощью:\n  /models --update {{model}} --api-key YOUR_API_KEY\n\nИли вручную добавьте файл ключа по пути: ~/.auto-coder/keys/{{model}}.key",
    },
    "async_model_check_error": {
        "en": "Error occurred while checking model '{{model}}': {{error}}",
        "zh": "检查模型 '{{model}}' 时发生错误: {{error}}",
        "ja": "モデル '{{model}}' の確認中にエラーが発生しました: {{error}}",
        "ar": "حدث خطأ أثناء فحص النموذج '{{model}}': {{error}}",
        "ru": "Ошибка при проверке модели '{{model}}': {{error}}",
    },
    "async_model_required": {
        "en": "Model parameter is required for async tasks.\n\nPlease specify a model using /model parameter or set default model in configuration.\n\nExample:\n  /async /name my_task /model gpt-4 implement user authentication",
        "zh": "异步任务需要模型参数。\n\n请使用 /model 参数指定模型或在配置中设置默认模型。\n\n示例:\n  /async /name my_task /model gpt-4 实现用户认证功能",
        "ja": "非同期タスクにはモデルパラメータが必要です。\n\n/model パラメータを使用してモデルを指定するか、設定でデフォルトモデルを設定してください。\n\n例:\n  /async /name my_task /model gpt-4 ユーザー認証を実装する",
        "ar": "معامل النموذج مطلوب للمهام غير المتزامنة.\n\nيرجى تحديد نموذج باستخدام معامل /model أو تعيين النموذج الافتراضي في التكوين.\n\nمثال:\n  /async /name my_task /model gpt-4 تنفيذ مصادقة المستخدم",
        "ru": "Для асинхронных задач требуется параметр модели.\n\nПожалуйста, укажите модель с помощью параметра /model или установите модель по умолчанию в конфигурации.\n\nПример:\n  /async /name my_task /model gpt-4 реализовать аутентификацию пользователя",
    },
    "async_time_param_format_error": {
        "en": "Invalid time format: {{time_str}}. Expected format: <number><unit>, where unit is one of: s (seconds), m (minutes), h (hours), d (days). Example: 5s, 10m, 2h, 1d",
        "zh": "时间参数格式错误: {{time_str}}。预期格式: <数字><单位>，单位必须是以下之一：s(秒)、m(分钟)、h(小时)、d(天)。示例：5s, 10m, 2h, 1d",
        "ja": "無効な時間形式: {{time_str}}。期待される形式: <数値><単位>、単位は以下のいずれかでなければなりません: s (秒)、m (分)、h (時間)、d (日)。例: 5s, 10m, 2h, 1d",
        "ar": "تنسيق الوقت غير صحيح: {{time_str}}. التنسيق المتوقع: <رقم><وحدة>، حيث تكون الوحدة واحدة من: s (ثواني)، m (دقائق)، h (ساعات)، d (أيام). مثال: 5s, 10m, 2h, 1d",
        "ru": "Неверный формат времени: {{time_str}}. Ожидаемый формат: <число><единица>, где единица является одной из: s (секунды), m (минуты), h (часы), d (дни). Пример: 5s, 10m, 2h, 1d",
    },
    "async_time_param_required": {
        "en": "The /time parameter is required for time-based execution.\n\nUsage: /async /name <task_name> /time <duration> <your_query>\n\nExample:\n  /async /name my_task /time 30m implement user authentication",
        "zh": "基于时间的执行必须提供 /time 参数。\n\n用法: /async /name <任务名称> /time <持续时间> <你的查询>\n\n示例:\n  /async /name my_task /time 30m 实现用户认证功能",
        "ja": "時間ベースの実行には /time パラメータが必要です。\n\n使用法: /async /name <タスク名> /time <期間> <クエリ>\n\n例:\n  /async /name my_task /time 30m ユーザー認証を実装する",
        "ar": "معامل /time مطلوب للتنفيذ القائم على الوقت.\n\nالاستخدام: /async /name <اسم_المهمة> /time <المدة> <استعلامك>\n\nمثال:\n  /async /name my_task /time 30m تنفيذ مصادقة المستخدم",
        "ru": "Параметр /time обязателен для выполнения на основе времени.\n\nИспользование: /async /name <имя_задачи> /time <продолжительность> <ваш_запрос>\n\nПример:\n  /async /name my_task /time 30m реализовать аутентификацию пользователя",
    },
    "async_task_name_conflict": {
        "en": "A task with the name '{{task_id}}' is already running.\n\nExisting task status: {{existing_status}}\nCreated at: {{existing_created}}\n\nPlease choose a different task name or wait for the existing task to complete.",
        "zh": "名为 '{{task_id}}' 的任务已在运行中。\n\n现有任务状态: {{existing_status}}\n创建时间: {{existing_created}}\n\n请选择不同的任务名称或等待现有任务完成。",
        "ja": "名前 '{{task_id}}' のタスクは既に実行中です。\n\n既存のタスクステータス: {{existing_status}}\n作成日時: {{existing_created}}\n\n別のタスク名を選択するか、既存のタスクが完了するまでお待ちください。",
        "ar": "مهمة باسم '{{task_id}}' قيد التشغيل بالفعل.\n\nحالة المهمة الحالية: {{existing_status}}\nتاريخ الإنشاء: {{existing_created}}\n\nيرجى اختيار اسم مهمة مختلف أو انتظار اكتمال المهمة الحالية.",
        "ru": "Задача с именем '{{task_id}}' уже выполняется.\n\nСтатус существующей задачи: {{existing_status}}\nСоздана: {{existing_created}}\n\nПожалуйста, выберите другое имя задачи или дождитесь завершения существующей задачи.",
    },
    "async_task_conflict_title": {
        "en": "Task Name Conflict",
        "zh": "任务名称冲突",
        "ja": "タスク名の競合",
        "ar": "تعارض اسم المهمة",
        "ru": "Конфликт имени задачи",
    },
    "async_workflow_unsupported_params": {
        "en": "In workflow mode, only /workflow and /name parameters are supported.\n\nUnsupported parameters found: {{params}}\n\nUsage: /async /workflow <workflow_name> /name <task_name> [query]\n\nExample:\n  /async /workflow my_workflow /name my_task",
        "zh": "在 workflow 模式下，仅支持 /workflow 和 /name 参数。\n\n发现不支持的参数: {{params}}\n\n用法: /async /workflow <工作流名称> /name <任务名称> [查询]\n\n示例:\n  /async /workflow my_workflow /name my_task",
        "ja": "ワークフローモードでは、/workflow と /name パラメータのみがサポートされています。\n\nサポートされていないパラメータが見つかりました: {{params}}\n\n使用法: /async /workflow <ワークフロー名> /name <タスク名> [クエリ]\n\n例:\n  /async /workflow my_workflow /name my_task",
        "ar": "في وضع سير العمل، يتم دعم معاملات /workflow و /name فقط.\n\nتم العثور على معاملات غير مدعومة: {{params}}\n\nالاستخدام: /async /workflow <اسم_سير_العمل> /name <اسم_المهمة> [استعلام]\n\nمثال:\n  /async /workflow my_workflow /name my_task",
        "ru": "В режиме рабочего процесса поддерживаются только параметры /workflow и /name.\n\nНайдены неподдерживаемые параметры: {{params}}\n\nИспользование: /async /workflow <имя_рабочего_процесса> /name <имя_задачи> [запрос]\n\nПример:\n  /async /workflow my_workflow /name my_task",
    },
}



"""
Auto Coder Terminal - 终端交互模块

这个模块提供了 auto-coder 的终端交互界面，包括：
- 命令处理和路由
- UI 组件（补全、键盘绑定、工具栏）
- 任务管理（异步任务、后台任务）
- 插件集成
"""

from .app import TerminalApp
from .bootstrap import run_cli

__all__ = ["TerminalApp", "run_cli"]



from typing import Union, Optional, Dict, Any, Type
from pydantic import Field
from ..llms import LLMManager
from .token_parser import TokenParser


class AutoCoderArgsParser:
    """AutoCoderArgs 参数解析器，提供高级参数解析功能"""
    
    def __init__(self, llm_manager: Optional[LLMManager] = None):
        """
        初始化参数解析器
        
        Args:
            llm_manager: LLM 管理器实例
        """
        self.llm_manager = llm_manager or LLMManager()
        self.token_parser = TokenParser(self.llm_manager)
    
    def parse_conversation_prune_safe_zone_tokens(self, 
                                                 value: Union[str, int, float],
                                                 code_model: Optional[str] = None) -> int:
        """
        解析 conversation_prune_safe_zone_tokens 参数
        
        Args:
            value: 参数值，支持字符串、整数、浮点数
            code_model: 代码模型名称，用于浮点数解析
            
        Returns:
            解析后的 token 数量
            
        Examples:
            >>> parser = AutoCoderArgsParser()
            >>> parser.parse_conversation_prune_safe_zone_tokens(51200)
            51200
            >>> parser.parse_conversation_prune_safe_zone_tokens("50k")
            51200
            >>> parser.parse_conversation_prune_safe_zone_tokens("50*1024")
            51200
            >>> parser.parse_conversation_prune_safe_zone_tokens(0.1, "deepseek/v3")
            # 返回 deepseek/v3 context_window 的 10%
        """
        try:
            return self.token_parser.parse_token_value(value, code_model)
        except ValueError as e:
            # 如果解析失败，返回默认值并记录警告
            default_value = 50 * 1024
            print(f"Warning: Failed to parse conversation_prune_safe_zone_tokens '{value}': {e}. Using default value: {default_value}")
            return default_value
    
    def parse_context_prune_safe_zone_tokens(self,
                                           value: Union[str, int, float],
                                           code_model: Optional[str] = None) -> int:
        """
        解析 context_prune_safe_zone_tokens 参数
        
        Args:
            value: 参数值
            code_model: 代码模型名称
            
        Returns:
            解析后的 token 数量
        """
        try:
            return self.token_parser.parse_token_value(value, code_model)
        except ValueError as e:
            default_value = 24 * 1024
            print(f"Warning: Failed to parse context_prune_safe_zone_tokens '{value}': {e}. Using default value: {default_value}")
            return default_value
    
    def parse_token_field(self, 
                         field_name: str,
                         value: Union[str, int, float],
                         code_model: Optional[str] = None,
                         default_value: Optional[int] = None) -> int:
        """
        通用的 token 字段解析方法
        
        Args:
            field_name: 字段名称
            value: 字段值
            code_model: 代码模型名称
            default_value: 默认值，如果为 None 则使用 50*1024
            
        Returns:
            解析后的 token 数量
        """
        try:
            return self.token_parser.parse_token_value(value, code_model)
        except ValueError as e:
            fallback_value = default_value or 50 * 1024
            print(f"Warning: Failed to parse {field_name} '{value}': {e}. Using default value: {fallback_value}")
            return fallback_value
    
    def validate_parsed_args(self, args_dict: Dict[str, Any], 
                           code_model: Optional[str] = None) -> Dict[str, Any]:
        """
        验证和解析整个参数字典
        
        Args:
            args_dict: 参数字典
            code_model: 代码模型名称
            
        Returns:
            验证和解析后的参数字典
        """
        # 定义需要解析的 token 字段及其默认值
        token_fields = {
            'conversation_prune_safe_zone_tokens': 50 * 1024,
            'context_prune_safe_zone_tokens': 24 * 1024,
            'context_prune_sliding_window_size': 1000,
            'context_prune_sliding_window_overlap': 100,
            'rag_params_max_tokens': 500000,
            'rag_context_window_limit': 120000,
            'rag_duckdb_vector_dim': 1024,
            'rag_duckdb_query_top_k': 10000,
            'rag_emb_dim': 1024,
            'rag_emb_text_size': 1024,
            'hybrid_index_max_output_tokens': 1000000,
            'data_cells_max_num': 2000,
        }
        
        parsed_dict = args_dict.copy()
        
        for field_name, default_value in token_fields.items():
            if field_name in parsed_dict:
                original_value = parsed_dict[field_name]
                # 如果值不是简单的 int，则进行解析
                if not isinstance(original_value, int):
                    parsed_dict[field_name] = self.parse_token_field(
                        field_name, original_value, code_model, default_value
                    )
        
        return parsed_dict
    
    def get_parsing_info(self) -> Dict[str, Any]:
        """
        获取解析器信息和使用示例
        
        Returns:
            包含解析器信息的字典
        """
        return {
            "supported_types": ["int", "float", "str"],
            "string_formats": self.token_parser.get_examples(),
            "supported_units": self.token_parser.get_supported_units(),
            "float_behavior": "Float values (0-1) are treated as percentages of the code_model's context_window",
            "fallback_context_window": 50 * 1024,
            "safety_notes": [
                "Invalid expressions fall back to default values",
                "Negative results are clamped to 0",
                "Division by zero is handled gracefully"
            ]
        }
    
    def create_flexible_field(self, 
                             default_value: Union[str, int, float],
                             description: str,
                             min_value: Optional[int] = None,
                             max_value: Optional[int] = None) -> Field:
        """
        创建支持灵活解析的 Pydantic 字段
        
        Args:
            default_value: 默认值
            description: 字段描述
            min_value: 最小值
            max_value: 最大值
            
        Returns:
            Pydantic Field 对象
        """
        field_kwargs = {
            "default": default_value,
            "description": description
        }
        
        if min_value is not None:
            field_kwargs["ge"] = min_value
        if max_value is not None:
            field_kwargs["le"] = max_value
            
        return Field(**field_kwargs) 


from autocoder.common.v2.agent.agentic_edit_tools.extra_docs.skill import _skill
from autocoder.common.v2.agent.agentic_edit_tools.extra_docs.workflow_subagents import (
    _workflow_subagents,
)
from autocoder.common.v2.agent.agentic_edit_tools.extra_docs.plan_format import (
    _plan_format,
)
from autocoder.common.v2.agent.agentic_edit_tools.extra_docs.interactive_session import (
    _interactive_session,
)
from autocoder.common.v2.agent.agentic_edit_tools.extra_docs.web_research import (
    _web_research,
)
from autocoder.common.v2.agent.agentic_edit_tools.extra_docs.ac_mod_files import (
    _ac_mod_files,
)

__all__ = [
    "_skill",
    "_workflow_subagents",
    "_plan_format",
    "_interactive_session",
    "_web_research",
    "_ac_mod_files",
]



from rich.console import Console
from rich.panel import Panel
from rich.text import Text
from typing import Optional,Dict,Any
from byzerllm.utils import format_str_jinja2
from autocoder.common.auto_coder_lang import get_message
from autocoder.chat_auto_coder_lang import get_message as get_chat_message

class Printer:
    def __init__(self,console:Optional[Console]=None):
        if console is None:
            self.console = Console()
        else:
            self.console = console

    def get_message_from_key(self, msg_key: str):
        try:
            v = get_message(msg_key)
            if not v:
                return get_chat_message(msg_key)
            else:
                return v
        except Exception as e:
            return get_chat_message(msg_key)

    def get_message_from_key_with_format(self, msg_key: str, **kwargs):        
        return format_str_jinja2(self.get_message_from_key(msg_key), **kwargs)
        
    def print_in_terminal(self, msg_key: str, style: str = None,**kwargs):     
        try:
            if style:                
                self.console.print(format_str_jinja2(self.get_message_from_key(msg_key),**kwargs), style=style)
            else:
                self.console.print(format_str_jinja2(self.get_message_from_key(msg_key),**kwargs))
        except Exception as e:
            try:
                print(self.get_message_from_key(msg_key))
            except Exception as e:
                print(f"text key: {msg_key} not found")

    
    def print_str_in_terminal(self, content: str, style: str = None):     
        try:
            if style:
                self.console.print(content, style=style)
            else:
                self.console.print(content)
        except Exception as e:
            print(content)   

    def print_panel(self, content: str, text_options:Dict[str,Any], panel_options:Dict[str,Any]):
        panel = Panel(Text(content, **text_options), **panel_options)
        self.console.print(panel)                  
        


from typing import Any, Dict, List, Optional, Union, Callable
from byzerllm.utils.client.types import (
    LLMFunctionCallResponse,
    LLMClassResponse, LLMResponse
)
import pydantic
from byzerllm import ByzerLLM
from byzerllm.utils.client import LLMResponse
from byzerllm.utils.types import SingleOutputMeta
from autocoder.rag.long_context_rag import LongContextRAG
from loguru import logger
from byzerllm.utils.langutil import asyncfy_with_semaphore


class LLWrapper:

    def __init__(self, llm: ByzerLLM, rag: Union[LongContextRAG]):
        self.llm = llm
        self.rag = rag

    def chat_oai(self,
                 conversations,
                 tools: List[Union[Callable, str]] = [],
                 tool_choice: Optional[Union[Callable, str]] = None,
                 execute_tool: bool = False,
                 impl_func: Optional[Callable] = None,
                 execute_impl_func: bool = False,
                 impl_func_params: Optional[Dict[str, Any]] = None,
                 func_params: Optional[Dict[str, Any]] = None,
                 response_class: Optional[Union[pydantic.BaseModel, str]] = None,
                 response_after_chat: Optional[Union[pydantic.BaseModel, str]] = False,
                 enable_default_sys_message: bool = True,
                 model: Optional[str] = None,
                 role_mapping=None,
                 llm_config: Dict[str, Any] = {},
                 only_return_prompt: bool = False,
                 extra_request_params: Dict[str, Any] = {}
                 ) -> Union[List[LLMResponse], List[LLMFunctionCallResponse], List[LLMClassResponse]]:
        res, contexts = self.rag.stream_chat_oai(
            conversations, llm_config=llm_config, extra_request_params=extra_request_params)
        metadata = {"request_id":""}
        output = ""
        for chunk in res:
            output += chunk[0]
            metadata["input_tokens_count"] = chunk[1].input_tokens_count
            metadata["generated_tokens_count"] = chunk[1].generated_tokens_count
            metadata["reasoning_content"] = chunk[1].reasoning_content
            metadata["finish_reason"] = chunk[1].finish_reason
            metadata["first_token_time"] = chunk[1].first_token_time
            
        return [LLMResponse(output=output, metadata=metadata, input="")]

    def stream_chat_oai(self, conversations,
                        model: Optional[str] = None,
                        role_mapping=None,
                        delta_mode=False,
                        llm_config: Dict[str, Any] = {},
                        extra_request_params: Dict[str, Any] = {}
                        ):
        res, contexts = self.rag.stream_chat_oai(
                conversations, llm_config=llm_config, extra_request_params=extra_request_params)

        if isinstance(res, tuple):
            for (t, metadata) in res:
                yield (t, SingleOutputMeta(
                    input_tokens_count=metadata.get("input_tokens_count", 0),
                    generated_tokens_count=metadata.get(
                        "generated_tokens_count", 0),
                    reasoning_content=metadata.get("reasoning_content", ""),
                    finish_reason=metadata.get("finish_reason", "stop"),
                    first_token_time=metadata.get("first_token_time", 0)
                ))
        else:
            for t in res:
                yield (t, SingleOutputMeta(0, 0))

    async def async_stream_chat_oai(self, conversations,
                                    model: Optional[str] = None,
                                    role_mapping=None,
                                    delta_mode=False,
                                    llm_config: Dict[str, Any] = {},
                                    extra_request_params: Dict[str, Any] = {}
                                    ):
        res, contexts = await asyncfy_with_semaphore(lambda: self.rag.stream_chat_oai(conversations, llm_config=llm_config, extra_request_params=extra_request_params))()
        # res,contexts = await self.llm.async_stream_chat_oai(conversations,llm_config=llm_config)
        for t in res:
            yield t

    def __getattr__(self, name):
        return getattr(self.llm, name)



"""Paste manager for handling paste file operations."""

import os
import uuid
from typing import Optional
from pathlib import Path


class PasteManager:
    """Manager for handling paste file operations."""
    
    def __init__(self, base_dir: Optional[str] = None):
        """Initialize paste manager.
        
        Args:
            base_dir: Base directory for paste files. If None, uses current working directory.
        """
        self.base_dir = base_dir or os.getcwd()
        self.paste_dir = os.path.join(self.base_dir, ".auto-coder", "pastes")
        self._ensure_paste_dir()
    
    def _ensure_paste_dir(self) -> None:
        """Ensure paste directory exists."""
        os.makedirs(self.paste_dir, exist_ok=True)
    
    def save_paste(self, content: str) -> str:
        """Save paste content to file and return file ID.
        
        Args:
            content: Content to save
            
        Returns:
            File ID (without path)
        """
        file_id = f"pasted-{uuid.uuid4().hex}"
        file_path = os.path.join(self.paste_dir, file_id)
        
        try:
            with open(file_path, "w", encoding="utf-8") as f:
                f.write(content)
            return file_id
        except Exception as e:
            raise RuntimeError(f"Failed to save paste content: {e}")
    
    def read_paste(self, file_id: str) -> Optional[str]:
        """Read paste content from file.
        
        Args:
            file_id: File ID to read
            
        Returns:
            File content or None if file not found
        """
        # Handle both "pasted-<uuid>" and just "<uuid>" formats
        if not file_id.startswith("pasted-"):
            file_id = f"pasted-{file_id}"
            
        file_path = os.path.join(self.paste_dir, file_id)
        
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                return f.read()
        except FileNotFoundError:
            return None
        except Exception as e:
            raise RuntimeError(f"Failed to read paste file {file_id}: {e}")
    
    def cleanup_old_pastes(self, max_age_hours: int = 24) -> int:
        """Clean up old paste files.
        
        Args:
            max_age_hours: Maximum age in hours for paste files
            
        Returns:
            Number of files cleaned up
        """
        import time
        
        if not os.path.exists(self.paste_dir):
            return 0
            
        current_time = time.time()
        max_age_seconds = max_age_hours * 3600
        cleaned_count = 0
        
        try:
            for filename in os.listdir(self.paste_dir):
                if filename.startswith("pasted-"):
                    file_path = os.path.join(self.paste_dir, filename)
                    if os.path.isfile(file_path):
                        file_age = current_time - os.path.getmtime(file_path)
                        if file_age > max_age_seconds:
                            os.remove(file_path)
                            cleaned_count += 1
        except Exception as e:
            raise RuntimeError(f"Failed to cleanup old pastes: {e}")
            
        return cleaned_count
    
    def list_pastes(self) -> list[str]:
        """List all paste file IDs.
        
        Returns:
            List of paste file IDs
        """
        if not os.path.exists(self.paste_dir):
            return []
            
        try:
            files = []
            for filename in os.listdir(self.paste_dir):
                if filename.startswith("pasted-") and os.path.isfile(
                    os.path.join(self.paste_dir, filename)
                ):
                    files.append(filename)
            return sorted(files)
        except Exception as e:
            raise RuntimeError(f"Failed to list paste files: {e}") 


import os
from typing import Optional
from autocoder.shadows.shadow_manager import ShadowManager
from autocoder.compilers.compiler_factory import CompilerFactory
from autocoder.compilers.models import ProjectCompilationResult,FileCompilationResult

class ShadowCompiler:
    """
    用于对ShadowManager管理的文件进行代码检查并将结果转换回项目路径的类。
    """
    
    def __init__(self, shadow_manager: ShadowManager,verbose:bool=False):
        """
        使用ShadowManager实例初始化。
        
        参数:
            shadow_manager (ShadowManager): 用于管理文件路径映射的实例
            verbose (bool): 是否启用详细输出
        """
        self.shadow_manager = shadow_manager
        self.compiler = CompilerFactory.create_compiler(language="provided",config_path=os.path.join(self.shadow_manager.source_dir,".auto-coder","projects","compiler.yml"))

    def compile_shadow_file(self, shadow_path: str) -> FileCompilationResult:
        return FileCompilationResult(
            file_path=shadow_path,
            success=True,
            language="provided",
            errors=[],
            error_message=None,
            warning_count=0,
            error_count=0,
            info_count=0,
            execution_time_ms=0,
            output_file=None
        )  

    def compile_all_shadow_files(self,target_compiler_name:Optional[str]=None) -> ProjectCompilationResult:
        link_projects_dir = self.shadow_manager.create_link_project()
        result = self.compiler.compile_project(link_projects_dir,target_compiler_name)
        result.project_path = self.shadow_manager.source_dir
        return result
        



"""
Conversation Message IDs Write Tool Resolver

This resolver handles writing conversation message IDs configurations for pruning.
It integrates with the pruner module to store message ID specifications along with
the current conversation ID, supporting create, append, and delete actions.
"""

import os
from typing import Optional
from autocoder.common.v2.agent.agentic_edit_tools.base_tool_resolver import BaseToolResolver
from autocoder.common.v2.agent.agentic_edit_types import ConversationMessageIdsWriteTool, ToolResult
from autocoder.common import AutoCoderArgs
from autocoder.common.pruner import get_conversation_message_ids_api,ConversationMessageIdsAPI
from autocoder.common.international import get_message
from loguru import logger
import typing

from autocoder.rag.lang import get_message_with_format

if typing.TYPE_CHECKING:
    from autocoder.common.v2.agent.agentic_edit import AgenticEdit


class ConversationMessageIdsWriteToolResolver(BaseToolResolver):
    """
    Resolver for the ConversationMessageIdsWriteTool.
    
    This resolver handles writing conversation message IDs configurations that specify
    which messages should be deleted during conversation pruning.
    Supports create, append, and delete actions.
    """
    
    def __init__(self, agent: 'AgenticEdit', tool: ConversationMessageIdsWriteTool, args: AutoCoderArgs):
        super().__init__(agent, tool, args)
        self.tool = tool

    def resolve(self) -> ToolResult:
        """
        Write conversation message IDs configuration.
        
        Returns:
            ToolResult with success status and configuration details
        """
        try:
            # Validate action parameter
            valid_actions = ["create", "append", "delete"]
            if self.tool.action not in valid_actions:
                return ToolResult(
                    success=False,
                    message=get_message_with_format("conversation_message_ids_invalid_action", action=self.tool.action),
                    content={
                        "error_type": "invalid_action",
                        "valid_actions": valid_actions,
                        "provided_action": self.tool.action
                    }
                )

            # Get current conversation ID            
            conversation_id = self.agent.conversation_config.conversation_id
            
            if not conversation_id:
                return ToolResult(
                    success=False,
                    message=get_message("conversation_message_ids_no_conversation"),
                    content={
                        "error_type": "no_conversation_id",
                        "message": get_message("conversation_message_ids_no_conversation")
                    }
                )
            
            # Get message IDs API
            message_ids_api = get_conversation_message_ids_api(
                storage_dir=self.args.range_storage_dir if hasattr(self.args, 'range_storage_dir') else None
            )
            
            # Handle different actions
            if self.tool.action == "create":
                return self._handle_create_action(message_ids_api, conversation_id)
            elif self.tool.action == "append":
                return self._handle_append_action(message_ids_api, conversation_id)
            elif self.tool.action == "delete":
                return self._handle_delete_action(message_ids_api, conversation_id)
            else:
                return ToolResult(
                    success=False,
                    message=get_message_with_format("conversation_message_ids_invalid_action", action=self.tool.action),
                    content={
                        "error_type": "unimplemented_action",
                        "action": self.tool.action
                    }
                )
                
        except Exception as e:
            error_msg = get_message_with_format("conversation_message_ids_operation_exception", error=str(e))
            logger.error(error_msg)
            return ToolResult(
                success=False,
                message=error_msg,
                content={
                    "error_type": "exception",
                    "error_message": str(e),
                    "tool_parameters": {
                        "message_ids": self.tool.message_ids,
                        "action": self.tool.action
                    }
                }
            )

    def _handle_create_action(self, message_ids_api, conversation_id: str) -> ToolResult:
        """Handle create action - create new configuration, replacing any existing one"""
        # Save message IDs configuration with default preserve_pairs=True
        success, error_msg, message_ids_obj = message_ids_api.save_conversation_message_ids(
            conversation_id=conversation_id,
            message_ids=self.tool.message_ids,
            description="Created via conversation_message_ids_write tool",
            preserve_pairs=True
        )
        
        if success and message_ids_obj:
            stats = message_ids_api.get_message_ids_statistics(conversation_id)
            
            return ToolResult(
                success=True,
                message=get_message("conversation_message_ids_create_success"),
                content={
                    "action": "create",
                    "conversation_id": conversation_id,
                    "message_ids": message_ids_obj.message_ids if message_ids_obj else [],
                    "description": message_ids_obj.description if message_ids_obj else "",
                    "preserve_pairs": message_ids_obj.preserve_pairs if message_ids_obj else True,
                    "statistics": stats,
                    "created_at": message_ids_obj.created_at if message_ids_obj else "",
                    "updated_at": message_ids_obj.updated_at if message_ids_obj else "",
                    "message": get_message("conversation_message_ids_create_success") + f"，共{len(message_ids_obj.message_ids) if message_ids_obj else 0}个消息ID。"
                }
            )
        else:
            logger.error(f"Failed to create conversation message IDs: {error_msg}")
            return ToolResult(
                success=False,
                message=get_message_with_format("conversation_message_ids_create_failed", error=error_msg),
                content={
                    "error_type": "create_failed",
                    "error_message": error_msg,
                    "conversation_id": conversation_id,
                    "message_ids": self.tool.message_ids
                }
            )

    def _handle_append_action(self, message_ids_api:ConversationMessageIdsAPI, conversation_id: str) -> ToolResult:
        """Handle append action - add to existing configuration"""
        # First, try to get existing configuration
        existing_config = message_ids_api.get_conversation_message_ids(conversation_id)
        
        if existing_config:
            # Parse existing message IDs
            existing_ids = set(existing_config.message_ids)
            # Parse new message IDs
            new_ids = set([id.strip() for id in self.tool.message_ids.split(",") if id.strip()])
            # Combine them
            combined_ids = existing_ids.union(new_ids)
            combined_ids_str = ",".join(sorted(combined_ids))
            
            # Save updated configuration
            success, error_msg, message_ids_obj = message_ids_api.save_conversation_message_ids(
                conversation_id=conversation_id,
                message_ids=combined_ids_str,
                description=existing_config.description + " | Appended via tool",
                preserve_pairs=existing_config.preserve_pairs
            )
        else:
            # No existing configuration, create new one
            success, error_msg, message_ids_obj = message_ids_api.save_conversation_message_ids(
                conversation_id=conversation_id,
                message_ids=self.tool.message_ids,
                description="Created via conversation_message_ids_write tool (append)",
                preserve_pairs=True
            )
        
        if success and message_ids_obj:
            stats = message_ids_api.get_message_ids_statistics(conversation_id)
            
            return ToolResult(
                success=True,
                message=get_message("conversation_message_ids_append_success"),
                content={
                    "action": "append",
                    "conversation_id": conversation_id,
                    "message_ids": message_ids_obj.message_ids if message_ids_obj else [],
                    "description": message_ids_obj.description if message_ids_obj else "",
                    "preserve_pairs": message_ids_obj.preserve_pairs if message_ids_obj else True,
                    "statistics": stats,
                    "created_at": message_ids_obj.created_at if message_ids_obj else "",
                    "updated_at": message_ids_obj.updated_at if message_ids_obj else "",
                    "message": get_message("conversation_message_ids_append_success") + f"，共{len(message_ids_obj.message_ids) if message_ids_obj else 0}个消息ID。"
                }
            )
        else:
            logger.error(f"Failed to append conversation message IDs: {error_msg}")
            return ToolResult(
                success=False,
                message=get_message_with_format("conversation_message_ids_append_failed", error=error_msg),
                content={
                    "error_type": "append_failed",
                    "error_message": error_msg,
                    "conversation_id": conversation_id,
                    "message_ids": self.tool.message_ids
                }
            )

    def _handle_delete_action(self, message_ids_api, conversation_id: str) -> ToolResult:
        """Handle delete action - remove from existing configuration"""
        # First, try to get existing configuration
        existing_config = message_ids_api.get_conversation_message_ids(conversation_id)
        
        if not existing_config:
            return ToolResult(
                success=False,
                message=get_message("conversation_message_ids_no_existing_config"),
                content={
                    "error_type": "no_existing_config",
                    "conversation_id": conversation_id,
                    "message": get_message("conversation_message_ids_no_existing_config")
                }
            )
        
        # Parse existing message IDs
        existing_ids = set(existing_config.message_ids)
        # Parse IDs to delete
        ids_to_delete = set([id.strip() for id in self.tool.message_ids.split(",") if id.strip()])
        # Remove them
        remaining_ids = existing_ids - ids_to_delete
        
        if not remaining_ids:
            # Delete the entire configuration if no IDs remain
            success, error_msg = message_ids_api.delete_conversation_message_ids(conversation_id)
            if success:
                return ToolResult(
                    success=True,
                    message=get_message("conversation_message_ids_delete_success"),
                    content={
                        "action": "delete",
                        "conversation_id": conversation_id,
                        "deleted_ids": list(ids_to_delete),
                        "remaining_ids": [],
                        "message": get_message("conversation_message_ids_delete_success")
                    }
                )
            else:
                return ToolResult(
                    success=False,
                    message=get_message_with_format("conversation_message_ids_delete_failed", error=error_msg),
                    content={
                        "error_type": "delete_failed",
                        "error_message": error_msg
                    }
                )
        else:
            # Update configuration with remaining IDs
            remaining_ids_str = ",".join(sorted(remaining_ids))
            success, error_msg, message_ids_obj = message_ids_api.save_conversation_message_ids(
                conversation_id=conversation_id,
                message_ids=remaining_ids_str,
                description=existing_config.description + " | Deleted IDs via tool",
                preserve_pairs=existing_config.preserve_pairs
            )
            
            if success and message_ids_obj:
                stats = message_ids_api.get_message_ids_statistics(conversation_id)
                
                return ToolResult(
                    success=True,
                    message=get_message("conversation_message_ids_update_success"),
                    content={
                        "action": "delete",
                        "conversation_id": conversation_id,
                        "deleted_ids": list(ids_to_delete),
                        "remaining_ids": message_ids_obj.message_ids if message_ids_obj else [],
                        "description": message_ids_obj.description if message_ids_obj else "",
                        "preserve_pairs": message_ids_obj.preserve_pairs if message_ids_obj else True,
                        "statistics": stats,
                        "created_at": message_ids_obj.created_at if message_ids_obj else "",
                        "updated_at": message_ids_obj.updated_at if message_ids_obj else "",
                        "message": get_message("conversation_message_ids_update_success") + f"，剩余{len(message_ids_obj.message_ids) if message_ids_obj else 0}个消息ID。"
                    }
                )
            else:
                logger.error(f"Failed to update conversation message IDs after delete: {error_msg}")
                return ToolResult(
                    success=False,
                    message=get_message_with_format("conversation_message_ids_update_failed", error=error_msg),
                    content={
                        "error_type": "update_after_delete_failed",
                        "error_message": error_msg,
                        "conversation_id": conversation_id,
                        "message_ids": self.tool.message_ids
                    }
                )



"""
Shell Commands Module

This module provides comprehensive shell command execution capabilities
including timeout control, process management, error recovery, and 
interactive session management with process type switching support.

Main Components:
- CommandExecutor: Main command execution interface
- TimeoutConfig, TimeoutManager: Timeout control system  
- ProcessManager: Process lifecycle management
- InteractiveCommandExecutor: Interactive command execution
- InteractiveProcess: Standard subprocess-based interactive process
- InteractivePexpectProcess: pexpect-based interactive process
- InteractiveSessionManager: Session management with process type switching
- Error recovery and process cleanup utilities

Usage:
    # Basic command execution
    from autocoder.common.shell_commands import CommandExecutor
    executor = CommandExecutor()
    result = executor.execute_command("ls -la")
    
    # Interactive sessions with process type control
    from autocoder.common.shell_commands import get_session_manager
    manager = get_session_manager()
    
    # Auto-select process type
    result = manager.create_session("python3")
    
    # Explicit process type selection
    result = manager.create_session("python3", use_pexpect=True)
    result = manager.create_session("echo test", use_pexpect=False)
"""

# Core command execution
from .command_executor import (
    CommandExecutor,
    execute_command,
    execute_command_generator,
    execute_command_background,
    execute_commands,
    get_background_processes,
    get_background_process_info,
    cleanup_background_process
)
from .exceptions import (
    CommandExecutionError,
    CommandTimeoutError,
    ProcessCleanupError
)

# Timeout management
from .timeout_config import TimeoutConfig
from .timeout_manager import TimeoutManager

# Process management
from .process_manager import ProcessManager
from .process_cleanup import cleanup_process_tree

# Interactive execution
from .interactive_executor import (
    InteractiveCommandExecutor, 
    InteractiveSession,
    execute_interactive,
    create_interactive_shell
)
from .interactive_process import InteractiveProcess

# pexpect-based execution
from .interactive_pexpect_process import (
    InteractivePexpectProcess, 
    PEXPECT_AVAILABLE
)

# Enhanced session management
from .interactive_session_manager import (
    InteractiveSessionManager,
    EnhancedSessionHandle,
    ProcessType,
    ToolResult,
    get_session_manager,
    clean_terminal_output
)

# Error recovery
from .error_recovery import ErrorRecoveryManager

# Monitoring
from .monitoring import CommandExecutionLogger, PerformanceMonitor
from .background_process_notifier import (
    BackgroundProcessNotifier,
    get_background_process_notifier,
    AsyncTaskMessage,
)

# Version information
__version__ = "1.2.0"

# Convenience exports
__all__ = [
    # Core execution
    "CommandExecutor",
    "execute_command",
    "execute_command_generator", 
    "execute_command_background",
    "execute_commands",
    "get_background_processes",
    "get_background_process_info",
    "cleanup_background_process",
    
    # Exceptions
    "CommandExecutionError",
    "CommandTimeoutError", 
    "ProcessCleanupError",
    
    # Timeout management
    "TimeoutConfig",
    "TimeoutManager",
    
    # Process management
    "ProcessManager",
    "cleanup_process_tree",
    
    # Interactive execution
    "InteractiveCommandExecutor",
    "InteractiveSession",
    "InteractiveProcess",
    "execute_interactive",
    "create_interactive_shell",
    
    # pexpect support
    "InteractivePexpectProcess",
    "PEXPECT_AVAILABLE",
    
    # Enhanced session management
    "InteractiveSessionManager",
    "EnhancedSessionHandle",
    "ProcessType",
    "ToolResult", 
    "get_session_manager",
    "clean_terminal_output",
    
    # Error recovery
    "ErrorRecoveryManager",
    
    # Monitoring
    "CommandExecutionLogger",
    "PerformanceMonitor",
    # Background notifier
    "BackgroundProcessNotifier",
    "get_background_process_notifier",
    "AsyncTaskMessage",
]


def get_available_process_types():
    """
    Get list of available process types.
    
    Returns:
        dict: Available process types with availability status
    """
    return {
        ProcessType.STANDARD: True,
        ProcessType.PEXPECT: PEXPECT_AVAILABLE
    }


def create_interactive_session(
    command: str,
    process_type: str = "auto",
    **kwargs
):
    """
    Convenience function to create an interactive session.
    
    Args:
        command: Command to execute
        process_type: "auto", "standard", "pexpect"
        **kwargs: Additional session arguments
        
    Returns:
        ToolResult with session information
    """
    manager = get_session_manager()
    
    if process_type == "auto":
        use_pexpect = None
    elif process_type == ProcessType.PEXPECT:
        use_pexpect = True
    elif process_type == ProcessType.STANDARD:
        use_pexpect = False
    else:
        raise ValueError(f"Invalid process type: {process_type}")
    
    return manager.create_session(command, use_pexpect=use_pexpect, **kwargs) 


"""
Tools Manager Core Implementation

工具管理器的核心实现，负责动态加载和管理 .autocodertools 目录中的工具命令。
"""

import os
import shutil
import platform
from pathlib import Path
from typing import List, Optional, Dict, Any
from loguru import logger
import byzerllm

from .models import ToolCommand, ToolsLoadResult
from .utils import is_tool_command_file, extract_tool_help, get_project_name
from ..priority_directory_finder import (
    PriorityDirectoryFinder,
    FinderConfig,
    SearchStrategy,
    ValidationMode,
)

MAX_TOOL_HELP_TEXT_DISPLAY_LENGTH = 100


def check_command_exists(command: str) -> bool:
    """
    检查命令是否存在于系统 PATH 中

    Args:
        command: 命令名称

    Returns:
        bool: 命令是否存在
    """
    return shutil.which(command) is not None


def get_dev_tools_status() -> Dict[str, bool]:
    """
    检测开发工具的安装状态

    Returns:
        Dict[str, bool]: 工具名称到是否可用的映射
    """
    return {
        "go": check_command_exists("go"),
        "bun": check_command_exists("bun"),
        "uv": check_command_exists("uv"),
    }


class ToolsManager:
    """
    工具管理器

    负责从多个优先级目录中加载和管理工具命令文件。
    支持的目录优先级（从高到低）：
    1. 当前项目/.autocodertools
    2. .auto-coder/.autocodertools
    3. ~/.auto-coder/.autocodertools
    4. ~/.auto-coder/.autocodertools/repos/<项目名>
    """

    def __init__(self, tools_dirs: Optional[List[str]] = None):
        """
        初始化工具管理器

        Args:
            tools_dirs: 自定义工具目录列表，如果为None则使用默认查找策略
        """
        self.tools_dirs = tools_dirs or self._find_tools_directories()
        self._result_cache: Optional[ToolsLoadResult] = None

    def _find_tools_directories(self) -> List[str]:
        """
        查找所有有效的工具目录

        Returns:
            List[str]: 有效的工具目录路径列表
        """

        config = FinderConfig(strategy=SearchStrategy.MERGE_ALL)

        # 添加标准目录路径
        current_dir = Path.cwd()
        project_name = get_project_name()

        # 1. 当前项目/.autocodertools (最高优先级)
        config.add_directory(
            str(current_dir / ".autocodertools"),
            priority=1,
            validation_mode=ValidationMode.HAS_FILES,
        )

        # 2. .auto-coder/.autocodertools
        config.add_directory(
            str(current_dir / ".auto-coder" / ".autocodertools"),
            priority=2,
            validation_mode=ValidationMode.HAS_FILES,
        )

        # 3. ~/.auto-coder/.autocodertools
        home_dir = Path.home()
        config.add_directory(
            str(home_dir / ".auto-coder" / ".autocodertools"),
            priority=3,
            validation_mode=ValidationMode.HAS_FILES,
        )

        # 4. ~/.auto-coder/.autocodertools/repos/<项目名> (最低优先级)
        config.add_directory(
            str(home_dir / ".auto-coder" / ".autocodertools" / "repos" / project_name),
            priority=4,
            validation_mode=ValidationMode.HAS_FILES,
        )

        finder = PriorityDirectoryFinder(config)
        result = finder.find_directories()

        if result.success:
            logger.info(f"找到工具目录: {result.all_valid_directories}")
            return result.all_valid_directories
        else:
            logger.warning("未找到任何工具目录")
            return []

    def load_tools(self, force_reload: bool = False) -> ToolsLoadResult:
        """
        加载所有工具命令

        Args:
            force_reload: 是否强制重新加载

        Returns:
            ToolsLoadResult: 加载结果
        """
        if not force_reload and self._result_cache is not None:
            return self._result_cache

        all_tools = []
        failed_count = 0

        for tools_dir in self.tools_dirs:
            if not os.path.exists(tools_dir):
                continue

            logger.debug(f"扫描工具目录: {tools_dir}")

            try:
                for item in os.listdir(tools_dir):
                    item_path = os.path.join(tools_dir, item)

                    if os.path.isfile(item_path) and is_tool_command_file(item_path):
                        try:
                            tool = self._create_tool_command(item_path, tools_dir)
                            if tool:
                                all_tools.append(tool)
                                logger.debug(f"加载工具: {tool.name}")
                            else:
                                failed_count += 1
                        except Exception as e:
                            logger.warning(f"加载工具文件失败 {item_path}: {e}")
                            failed_count += 1

            except OSError as e:
                logger.warning(f"读取工具目录失败 {tools_dir}: {e}")
                continue

        # 去重：如果多个目录中有同名工具，优先使用高优先级目录中的
        unique_tools = self._deduplicate_tools(all_tools)

        result = ToolsLoadResult(
            success=True, tools=unique_tools, failed_count=failed_count
        )
        self._result_cache = result

        return result

    def _create_tool_command(
        self, file_path: str, source_dir: str
    ) -> Optional[ToolCommand]:
        """
        创建工具命令对象

        Args:
            file_path: 工具文件路径
            source_dir: 来源目录

        Returns:
            Optional[ToolCommand]: 工具命令对象
        """
        path = Path(file_path)

        # 提取帮助信息
        help_text = extract_tool_help(file_path)

        # 检查是否可执行
        is_executable = os.access(file_path, os.X_OK)

        # 检查是否存在对应的 .key 文件
        # 推荐命名: <tool_name>.key（不含扩展名）
        key_file_path = path.with_suffix(".key")
        # 兼容旧命名: <tool_filename>.<ext>.key（例如 deploy.sh.key）
        legacy_key_file_path = path.with_suffix(path.suffix + ".key")
        has_key_file = key_file_path.exists() or legacy_key_file_path.exists()

        return ToolCommand(
            name=path.stem,
            path=file_path,
            help_text=help_text,
            is_executable=is_executable,
            file_extension=path.suffix,
            source_directory=source_dir,
            has_key_file=has_key_file,
        )

    def _deduplicate_tools(self, tools: List[ToolCommand]) -> List[ToolCommand]:
        """
        去重工具列表，保留高优先级目录中的工具

        Args:
            tools: 工具列表

        Returns:
            List[ToolCommand]: 去重后的工具列表
        """
        # 创建目录优先级映射
        dir_priority = {dir_path: idx for idx, dir_path in enumerate(self.tools_dirs)}

        # 按工具名称分组
        tools_by_name: Dict[str, List[ToolCommand]] = {}
        for tool in tools:
            if tool.name not in tools_by_name:
                tools_by_name[tool.name] = []
            tools_by_name[tool.name].append(tool)

        # 对每个工具名称，选择优先级最高的工具
        unique_tools = []
        for name, tool_list in tools_by_name.items():
            if len(tool_list) == 1:
                unique_tools.append(tool_list[0])
            else:
                # 选择优先级最高的工具
                best_tool = min(
                    tool_list, key=lambda t: dir_priority.get(t.source_directory, 999)
                )
                unique_tools.append(best_tool)

                # 记录被覆盖的工具
                for tool in tool_list:
                    if tool != best_tool:
                        logger.debug(
                            f"工具 {name} 在 {tool.source_directory} 被 {best_tool.source_directory} 中的版本覆盖"
                        )

        return unique_tools

    def get_tool_by_name(self, name: str) -> Optional[ToolCommand]:
        """
        根据名称获取工具命令

        Args:
            name: 工具名称

        Returns:
            Optional[ToolCommand]: 工具命令对象
        """
        result = self.load_tools()
        if not result.success:
            return None

        for tool in result.tools:
            if tool.name == name:
                return tool

            # 兼容：允许传入带扩展名的名称（如 deploy.sh）
            full_name = (
                f"{tool.name}{tool.file_extension}"
                if tool.file_extension
                else tool.name
            )
            if full_name == name:
                return tool
        return None

    def list_tool_names(self) -> List[str]:
        """
        获取所有工具名称列表

        Returns:
            List[str]: 工具名称列表
        """
        result = self.load_tools()
        if not result.success:
            return []
        return [tool.name for tool in result.tools]

    @byzerllm.prompt()
    def get_tools_prompt(self) -> str:
        """
        # Available External(or Custom) Tool Commands

        Project Name: {{ project_name }}
        Current Project Path: {{ project_path }}
        Total Tools: {{ tools_count }}
        {% if failed_count > 0 %}
        Failed to Load: {{ failed_count }} tools
        {% endif %}

        ## Tool Directories
        {% for dir in tools_directories %}
        - {{ dir }}
        {% endfor %}

        ## Tool List

        {% if tools_count == 0 %}
        No available tool commands found.
        {% elif tools_count > 5 %}
        **Note**: There are {{ tools_count }} tools available. To avoid prompt bloat, detailed info is not listed here.

        **How to discover and use tools**:
        1. List tool directories above with `list_files` (preferred). If a directory is ignored by `list_files`/`search_files`
           (e.g., under `.auto-coder`), use `execute_command` to run `ls <tool_directory>` / `ls -la <tool_directory>`.
        2. View tool usage: `<tool_name> --help` or `<tool_name> -h`
        3. View full skill document (if supported): `<tool_name> --skill`
        4. Prefer `execute_external_tool` to run tools (especially when the tool has a `.key` file for credentials injection).

        **Quick Reference** (tool names only):
        {% for tool in tools_info %}
        - `{{ tool.name }}`{% if tool.file_extension %} ({{ tool.file_extension }}){% endif %}{% if tool.has_key_file %} (has .key file, credentials auto-injected){% endif %}
        {% endfor %}
        {% else %}
        {% for tool in tools_info %}
        ### {{ tool.name }}{{ tool.file_extension }}

        **Source Directory**: {{ tool.source_directory }}
        **Executable**: {% if tool.is_executable %}Yes{% else %}No{% endif %}
        **Has .key File**: {% if tool.has_key_file %} Yes (credentials auto-injected, DO NOT read){% else %} No{% endif %}

        **Usage Instructions**:
        ```
        {{ tool.help_text }}
        ```

        ---
        {% endfor %}
        {% endif %}

        ## How to Create External Tools

        ### Directory Structure (Priority Order)
        Tools are loaded from these directories in priority order (highest to lowest):
        1. **Project-specific**: `./.autocodertools/` (**recommended**)
        2. **Project config**: `./.auto-coder/.autocodertools/`
        3. **Global user**: `~/.auto-coder/.autocodertools/`
        4. **Project-specific global**: `~/.auto-coder/.autocodertools/repos/{{ project_name }}/` (lowest priority)

        **Note**:
        - Tools with identical names in higher priority directories will override those in lower priority directories.
        - The source code of the tools is in the directory `{{ autocoder_home }}/.auto-coder/tool_repos/`.
        - To inspect tools or tool_repos content, use `execute_command` tool (e.g., `ls`) since the `.auto-coder` directory is excluded from `search_files` results.

        ### Supported File Types
        - **Executable binaries** (compiled tools, **recommended**)
        - **Script files** (`.sh`, `.py`, `.js`, `.rb`, etc.)

        ### Tool Development Guidelines

        Please use SubAgent to create new tools. Before invoke the subagent, you should create the project directory in {{ autocoder_home }}/.auto-coder/tool_repos/ first
        and then go in to the project directory and then echo '<prompt>' | auto-coder.run --model <model_name> --is-sub-agent to invoke the subagent.
        New tools should follow the same guidelines as existing tools.

        #### 1. Help Information (Required)
        Your tool must provide help information using one of these methods:

        **Method 1: Command-line help (Recommended)**
        ```bash
        your_tool help
        your_tool -h
        your_tool --help
        ```

        **Notes**:
        - `your_tool --help` can be used to view CLI help for most tools.
        - `your_tool --skill` can be used to view the full skill document of the command, but this parameter is optional and
          not all tools provide it.

        **Method 2: File header comments**
        ```python
        #!/usr/bin/env python3
        # Description: Brief description of your tool
        # Usage: tool_name [options] [arguments]
        #
        # Options:
        #   -h, --help     Show this help message
        #   -v, --verbose  Enable verbose output
        ```

        #### 2. File Permissions
        - **Binary files** (no extension): Must be executable (`chmod +x`)
        - **Script files** (`.py`, `.sh`, etc.): Must be readable (`chmod +r`)

        #### 3. Tool Execution
        Use the built-in `execute_external_tool` tool to run your custom tools.
        If the tool requires sensitive credentials (accounts/passwords/API keys), you must use `execute_external_tool` instead.

        #### 3.1 Credentials / Sensitive Parameters (Recommended)
        Some external tools require sensitive information (e.g., `api_key`, `token`, `username`, `password`), and different
        subcommands may require different credentials.

        - Store sensitive values in a YAML file named `<tool_name>.key` (same directory as the tool file).
        - This `.key` file supports `global` params and nested multi-level `subcommands` configuration.
        - The `execute_external_tool` tool will automatically inject these params into the command line, and will mask
          sensitive values in display/logs.

        **⚠️ IMPORTANT: If a `.key` file exists for the tool, you DO NOT read the key file and DO NOT need to pass credentials (accounts/passwords/API keys)
        when calling the tool. The system will automatically inject them from the `.key` file. Just call the tool with
        other required parameters only.**

        Example `.key` file (supports multi-level subcommands):
        ```yaml
        global:
          api_key: "sk-xxxx"
          username: "user"

        # Multi-level subcommands with _params for each level
        subcommands:
          config:
            _params:              # params for "config" subcommand
              verbose: "true"
            set:
              _params:            # params for "config,set" subcommands
                format: "json"
              proxy:
                _params:          # params for "config,set,proxy" subcommands
                  timeout: "30"
          upload:
            _params:
              token: "yyy"

        # Optional formatting settings
        param_style: "space"      # "space" => --key value, "equals" => --key=value
        param_prefix: "--"        # or "-"
        ```

        **Usage Example**:
        - With `.key` file: `tool_name config set proxy --url http://example.com` (credentials auto-injected)
        - Multi-level subcommands: Use comma-separated format like `<subcommands>config,set,proxy</subcommands>`

        #### 4. **Tool is now available** - the AI assistant will discover and use it freely

        ## Important Rules

        ### Rule 1: NEVER Read .key Files (CRITICAL SECURITY RULE)
        ⚠️ **ABSOLUTELY FORBIDDEN**: You are **STRICTLY PROHIBITED** from reading, viewing, or accessing any `.key` files.
        - `.key` files contain sensitive credentials (API keys, passwords, tokens)
        - The system automatically injects credentials from `.key` files when executing tools
        - If a tool has a `.key` file (marked with ✅), you MUST NOT attempt to read it
        - Violation of this rule is a **SECURITY BREACH**

        ### Rule 2: Help Parameter (MANDATORY)
        All tools MUST support `help` or `-h` parameter for detailed usage information.

        ### Rule 3: Use SubAgent to Create Tools (REQUIRED)
        - **NEVER** create tools directly in the current agent
        - **ALWAYS** use a sub agent with proper timeout (e.g., 1800s)

        ### Rule 4: Preferred Language Priority

        **Development Tools Status**:
        | Priority | Language | Package Manager | Status |
        |----------|----------|-----------------|--------|
        | 1st      | **Go**   | go mod          | {% if dev_tools.go %}✅ Available{% else %}❌ Not Installed{% endif %} |
        | 2nd      | Node.js  | bun             | {% if dev_tools.bun %}✅ Available{% else %}❌ Not Installed{% endif %} |
        | 3rd      | Python   | uv              | {% if dev_tools.uv %}✅ Available{% else %}❌ Not Installed{% endif %} |

        {% if not dev_tools.go and not dev_tools.bun and not dev_tools.uv %}
        ⚠️ **WARNING**: No development tools detected! Please install Go first (recommended).
        {% elif not dev_tools.go %}
        💡 **Recommendation**: Go is not installed. Consider installing Go for better tool development experience.
        {% endif %}

        {% if not dev_tools.go %}
        ### Go Installation Guide

        **{{ os_type }}**:
        {% if os_type == "macOS" %}
        ```bash
        # Using Homebrew (recommended)
        brew install go

        # Or download from official website
        # https://go.dev/dl/
        ```
        {% elif os_type == "Linux" %}
        ```bash
        # Ubuntu/Debian
        sudo apt update && sudo apt install golang-go

        # Or using snap
        sudo snap install go --classic

        # Or download from official website
        # https://go.dev/dl/
        ```
        {% elif os_type == "Windows" %}
        ```powershell
        # Using winget (recommended)
        winget install GoLang.Go

        # Using Chocolatey
        choco install golang

        # Or download from official website
        # https://go.dev/dl/
        ```
        {% endif %}

        After installation, verify with:
        ```bash
        go version
        ```
        {% endif %}

        ### Rule 5: Tool Creation Workflow (Step-by-Step)

        **Step 1**: Create project directory

        Linux/macOS:
        ```bash
        mkdir -p {{ autocoder_home }}/.auto-coder/tool_repos/<tool_name>
        ```

        Windows (PowerShell):
        ```powershell
        New-Item -ItemType Directory -Force -Path "{{ autocoder_home }}\.auto-coder\tool_repos\<tool_name>"
        ```

        **Step 2**: Invoke SubAgent to develop the tool

        Linux/macOS:
        ```bash
        cd {{ autocoder_home }}/.auto-coder/tool_repos/<tool_name> && echo '<your_prompt>' | auto-coder.run --model <model_name> --is-sub-agent
        ```

        Windows (PowerShell):
        ```powershell
        cd "{{ autocoder_home }}\.auto-coder\tool_repos\<tool_name>"; echo '<your_prompt>' | auto-coder.run --model <model_name> --is-sub-agent
        ```

        **Step 3**: Copy the compiled binary to tools directory

        Linux/macOS:
        ```bash
        cp {{ autocoder_home }}/.auto-coder/tool_repos/<tool_name>/<binary_name> {{ project_path }}/.autocodertools/
        ```

        Windows (PowerShell):
        ```powershell
        Copy-Item "{{ autocoder_home }}\.auto-coder\tool_repos\<tool_name>\<binary_name>.exe" "{{ project_path }}\.autocodertools\"
        ```

        **Step 4**: Verify
        - If no binary found, ask SubAgent to rebuild
        - Test the tool with `<tool_name> --help` (Linux/macOS) or `<tool_name>.exe --help` (Windows)
        """

        # 加载所有工具
        result = self.load_tools()

        # 检测开发工具状态
        dev_tools = get_dev_tools_status()

        # 检测操作系统类型
        system = platform.system()
        if system == "Darwin":
            os_type = "macOS"
        elif system == "Windows":
            os_type = "Windows"
        else:
            os_type = "Linux"

        if not result.success:
            return {
                "project_name": get_project_name(),
                "project_path": os.getcwd(),
                "autocoder_home": os.path.expanduser("~"),
                "tools_count": 0,
                "tools_info": [],
                "failed_count": 0,
                "tools_directories": self.tools_dirs,
                "error_message": result.error_message or "未找到任何工具",
                "dev_tools": dev_tools,
                "os_type": os_type,
            }  # type: ignore[return]

        tools_info = []
        for tool in result.tools:
            help_text = tool.help_text
            if len(help_text) > MAX_TOOL_HELP_TEXT_DISPLAY_LENGTH:
                help_text = help_text[:MAX_TOOL_HELP_TEXT_DISPLAY_LENGTH]
            tools_info.append(
                {
                    "name": tool.name,
                    "help_text": help_text,
                    "is_executable": tool.is_executable,
                    "file_extension": tool.file_extension,
                    "source_directory": tool.source_directory,
                    "has_key_file": tool.has_key_file,
                }
            )

        project_name = get_project_name()

        return {
            "project_name": project_name,
            "project_path": os.getcwd(),
            "autocoder_home": os.path.expanduser("~"),
            "tools_count": len(result.tools),
            "tools_info": tools_info,
            "failed_count": result.failed_count,
            "tools_directories": self.tools_dirs,
            "dev_tools": dev_tools,
            "os_type": os_type,
        }  # type: ignore[return]



import byzerllm
from autocoder.common import AutoCoderArgs
from typing import Dict
from byzerllm.utils.client import code_utils
import json
import base64
import platform
import matplotlib.font_manager as fm
from pydantic import BaseModel
class LogoDesign(BaseModel):
    selectedStyle: str
    companyName: str
    selectedBackgroundColor: str
    selectedPrimaryColor: str
    additionalInfo: str

class LogoDesigner:
    def __init__(self, args: AutoCoderArgs, llm: byzerllm.ByzerLLM):
        self.llm = llm
        if not args.sd_model:
            raise ValueError("sd_model is not set")
        self.sd_model_llm = self.llm.get_sub_client("sd_model")
        self.args = args
        
    @byzerllm.prompt()
    def extract_logo_info(self, query: str) -> str:
        """
        根据用户的需求，抽取相关信息，生成一个LogoDesign对象。生成的信息必须使用英文，对于没有提及的信息，请
        根据你对用户需求的理解，给出合理的默认值。

        用户需求:
        我想创建一个闪亮前卫的科技公司logo，我的公司名叫做"ByteWave"，喜欢深蓝色和白色的搭配。我们是一家专注于人工智能的公司。

        LogoDesign对象示例:

        ```json
        {
            "selectedStyle": "Tech",
            "companyName": "ByteWave", 
            "selectedBackgroundColor": "white",
            "selectedPrimaryColor": "dark blue",
            "additionalInfo": "AI technology focused company"
        }
        ```

        现在请根据如下用户需求生成一个LogoDesign Json对象:

        {{ query }}
        """
    
    @byzerllm.prompt()
    def enhance_logo_generate(
        self,
        selectedStyle:str,
        companyName,
        selectedBackgroundColor,
        selectedPrimaryColor,
        additionalInfo,
    ) -> str:
        """
        A single logo, high-quality, award-winning professional design, made for both digital and print media, only contains a few vector shapes, {{ selectedStyle }}

        Primary color is {{ selectedPrimaryColor }} and background color is {{ selectedBackgroundColor }}. The company name is {{ companyName }}, make sure to include the company name in the logo. {{ "Additional info: " + additionalInfo if additionalInfo else "" }}
        """
        style_lookup = {
            "Flashy": "Flashy, attention grabbing, bold, futuristic, and eye-catching. Use vibrant neon colors with metallic, shiny, and glossy accents.",
            "Tech": "highly detailed, sharp focus, cinematic, photorealistic, Minimalist, clean, sleek, neutral color pallete with subtle accents, clean lines, shadows, and flat.",
            "Modern": "modern, forward-thinking, flat design, geometric shapes, clean lines, natural colors with subtle accents, use strategic negative space to create visual interest.",
            "Playful": "playful, lighthearted, bright bold colors, rounded shapes, lively.",
            "Abstract": "abstract, artistic, creative, unique shapes, patterns, and textures to create a visually interesting and wild logo.",
            "Minimal": "minimal, simple, timeless, versatile, single color logo, use negative space, flat design with minimal details, Light, soft, and subtle.",
        }
        return {"selectedStyle": style_lookup[selectedStyle],"additionalInfo":self.enhance_query.prompt(additionalInfo)}

    @byzerllm.prompt()
    def enhance_query(self, query: str) -> str:
        """
        你非常擅长使用文生图模型，特别能把用户简单的需求具象化。你的目标是转化用户的需求，使得转化后的
        文本更加适合问生图模型生成符合用户需求的图片。

        特别注意：
        1. 无论用户使用的是什么语言，你的改进后的表达都需要是英文。

        用户需求：
        我想设计一个偏卡通类的游戏，AGI PoLang 的游戏应用界面。

        改进后的表达：

        ```text
        Design a vibrant game app interface titled 'AGI PoLang' with a soothing blue background featuring a lush landscape of towering trees and dense bushes framing the sides.
        At the heart of the screen, elegantly display the game's name in a golden,
        playful font that captivates attention.
        Beneath this, strategically place two interactive buttons labeled 'Play Game' and 'Quit Game',
        ensuring they are both visually appealing and user-friendly.
        Center stage, introduce an engaging scene where an orange wolf stands majestically on a verdant field,
        set against a serene blue sky dotted with fluffy white clouds.
        The entire design should radiate a fun and playful atmosphere,
        encapsulating the spirit of adventure and joy that 'AGI PoLang' promises to deliver.
        ```

        用户需求：
        帮我设计一个财务报告网页页面，要时尚美观。

        改进后的表达：
        ```text
        Create a web UI page. Design a sleek and intuitive financial reports web page,
        featuring a clean menu layout, seamless navigation,
        and a comprehensive display of reports.
        The website should embody a clear and organized typography, enhancing readability and user experience.
        Incorporate dynamic lighting effects to highlight key financial data, simulating the precision and clarity of a well-managed budget.
        The overall design should reflect the stability and growth of financial health,
        serving as a visual metaphor for the solid foundation and strategic planning required in personal and corporate finance.
        ```

        用户需求：

        生成一个在自然环境中展示的防晒产品的逼真模型。产品应置于画面中心，标签清晰可见。
        将产品置于一种原始风景的背景之前，该风景包括山脉、湖泊和生动的绿色植被。
        包括花朵、石头，可能还有一两只鹿在背景中，以增强户外、新鲜的感觉。
        使用明亮的自然光照强调产品，并确保构图平衡，色彩搭配和谐，与品牌设计相得益彰。

        改进后的表达：
        ```text
        Generate a photorealistic mockup of a sunscreen product displayed in a serene natural setting.
        The product should be centered in the frame, with its label clearly visible.
        Position the product against a backdrop of a pristine landscape featuring a mountain, a lake, and vibrant greenery.
        Include natural elements like flowers, rocks, and possibly a deer or two in the background to enhance the outdoor, fresh feel.
        Use bright, natural lighting to highlight the product, and ensure the composition is balanced with a harmonious color palette
        that complements the brand's design.
        ```

        用户需求:
        一个扎着双马尾、手持棒球棒的女人，走在走廊上。灯光转变为危险的红色，营造出紧张的黑色电影风格氛围。

        改进后的表达：
        ```text
        A slow dolly-in camera follows a woman with pigtails holding a baseball bat as she walks down a dimly lit hallway.
        The lighting shifts to a menacing red, creating a tense, noir-style atmosphere with echoing footsteps adding suspense.
        ```

        现在让我们开始一个新的任务。

        用户需求：
        {{ query }}

        改进后的表达（改进后的表达请用 ```text ```进行包裹）：
        """

    def run(self, query: str):
        logo_design = self.extract_logo_info.with_llm(self.llm).with_return_type(LogoDesign).run(query)
    
        enhanced_query_str = (
            self.enhance_logo_generate.with_llm(self.llm)
            .with_extractor(lambda x: code_utils.extract_code(x)[0][1])
            .run(selectedStyle = logo_design.selectedStyle,
                 companyName = logo_design.companyName,
                 selectedBackgroundColor = logo_design.selectedBackgroundColor,
                 selectedPrimaryColor = logo_design.selectedPrimaryColor,
                 additionalInfo = logo_design.additionalInfo)
        )        
        response = self.sd_model_llm.chat_oai(
            conversations=[
                {
                    "role": "user",
                    "content": json.dumps(
                        {"input": enhanced_query_str, "size": "1024x1024"},
                        ensure_ascii=False,
                    ),
                }
            ]
        )

        image_data = base64.b64decode(response[0].output)
        with open("output.jpg", "wb") as f:
            f.write(image_data)    

class SDDesigner:
    def __init__(self, args: AutoCoderArgs, llm: byzerllm.ByzerLLM):
        self.llm = llm
        if not args.sd_model:
            raise ValueError("sd_model is not set")
        self.sd_model_llm = self.llm.get_sub_client("sd_model")
        self.args = args

    @byzerllm.prompt()
    def enhance_logo_generate(
        self,
        selectedStyle:str,
        companyName,
        selectedBackgroundColor,
        selectedPrimaryColor,
        additionalInfo,
    ) -> str:
        """
        A single logo, high-quality, award-winning professional design, made for both digital and print media, only contains a few vector shapes, {{ selectedStyle }}

        Primary color is {{ selectedPrimaryColor }} and background color is {{ selectedBackgroundColor }}. The company name is {{ companyName }}, make sure to include the company name in the logo. {{ "Additional info: " + additionalInfo if additionalInfo else "" }}
        """
        style_lookup = {
            "Flashy": "Flashy, attention grabbing, bold, futuristic, and eye-catching. Use vibrant neon colors with metallic, shiny, and glossy accents.",
            "Tech": "highly detailed, sharp focus, cinematic, photorealistic, Minimalist, clean, sleek, neutral color pallete with subtle accents, clean lines, shadows, and flat.",
            "Modern": "modern, forward-thinking, flat design, geometric shapes, clean lines, natural colors with subtle accents, use strategic negative space to create visual interest.",
            "Playful": "playful, lighthearted, bright bold colors, rounded shapes, lively.",
            "Abstract": "abstract, artistic, creative, unique shapes, patterns, and textures to create a visually interesting and wild logo.",
            "Minimal": "minimal, simple, timeless, versatile, single color logo, use negative space, flat design with minimal details, Light, soft, and subtle.",
        }
        return {"selectedStyle": style_lookup[selectedStyle],"additionalInfo":self.enhance_query.prompt(additionalInfo)}

    @byzerllm.prompt()
    def enhance_query(self, query: str) -> str:
        """
        你非常擅长使用文生图模型，特别能把用户简单的需求具象化。你的目标是转化用户的需求，使得转化后的
        文本更加适合问生图模型生成符合用户需求的图片。

        特别注意：
        1. 无论用户使用的是什么语言，你的改进后的表达都需要是英文。

        用户需求：
        我想设计一个偏卡通类的游戏，AGI PoLang 的游戏应用界面。

        改进后的表达：

        ```text
        Design a vibrant game app interface titled 'AGI PoLang' with a soothing blue background featuring a lush landscape of towering trees and dense bushes framing the sides.
        At the heart of the screen, elegantly display the game's name in a golden,
        playful font that captivates attention.
        Beneath this, strategically place two interactive buttons labeled 'Play Game' and 'Quit Game',
        ensuring they are both visually appealing and user-friendly.
        Center stage, introduce an engaging scene where an orange wolf stands majestically on a verdant field,
        set against a serene blue sky dotted with fluffy white clouds.
        The entire design should radiate a fun and playful atmosphere,
        encapsulating the spirit of adventure and joy that 'AGI PoLang' promises to deliver.
        ```

        用户需求：
        帮我设计一个财务报告网页页面，要时尚美观。

        改进后的表达：
        ```text
        Create a web UI page. Design a sleek and intuitive financial reports web page,
        featuring a clean menu layout, seamless navigation,
        and a comprehensive display of reports.
        The website should embody a clear and organized typography, enhancing readability and user experience.
        Incorporate dynamic lighting effects to highlight key financial data, simulating the precision and clarity of a well-managed budget.
        The overall design should reflect the stability and growth of financial health,
        serving as a visual metaphor for the solid foundation and strategic planning required in personal and corporate finance.
        ```

        用户需求：

        生成一个在自然环境中展示的防晒产品的逼真模型。产品应置于画面中心，标签清晰可见。
        将产品置于一种原始风景的背景之前，该风景包括山脉、湖泊和生动的绿色植被。
        包括花朵、石头，可能还有一两只鹿在背景中，以增强户外、新鲜的感觉。
        使用明亮的自然光照强调产品，并确保构图平衡，色彩搭配和谐，与品牌设计相得益彰。

        改进后的表达：
        ```text
        Generate a photorealistic mockup of a sunscreen product displayed in a serene natural setting.
        The product should be centered in the frame, with its label clearly visible.
        Position the product against a backdrop of a pristine landscape featuring a mountain, a lake, and vibrant greenery.
        Include natural elements like flowers, rocks, and possibly a deer or two in the background to enhance the outdoor, fresh feel.
        Use bright, natural lighting to highlight the product, and ensure the composition is balanced with a harmonious color palette
        that complements the brand's design.
        ```

        用户需求:
        一个扎着双马尾、手持棒球棒的女人，走在走廊上。灯光转变为危险的红色，营造出紧张的黑色电影风格氛围。

        改进后的表达：
        ```text
        A slow dolly-in camera follows a woman with pigtails holding a baseball bat as she walks down a dimly lit hallway.
        The lighting shifts to a menacing red, creating a tense, noir-style atmosphere with echoing footsteps adding suspense.
        ```

        现在让我们开始一个新的任务。

        用户需求：
        {{ query }}

        改进后的表达（改进后的表达请用 ```text ```进行包裹）：
        """

    def run(self, query: str):
        enhanced_query_str = (
            self.enhance_query.with_llm(self.llm)
            .with_extractor(lambda x: code_utils.extract_code(x)[0][1])
            .run(query)
        )
        print(enhanced_query_str)
        response = self.sd_model_llm.chat_oai(
            conversations=[
                {
                    "role": "user",
                    "content": json.dumps(
                        {"input": enhanced_query_str, "size": "1024x1024"},
                        ensure_ascii=False,
                    ),
                }
            ]
        )

        image_data = base64.b64decode(response[0].output)
        with open("output.jpg", "wb") as f:
            f.write(image_data)


class SVGDesigner:
    def __init__(self, args: AutoCoderArgs, llm: byzerllm.ByzerLLM):
        self.llm = llm
        if args.designer_model:
            self.llm = self.llm.get_sub_client("designer_model")
        self.args = args
        self.system_info = self.get_system_info()

    def get_system_info(self):
        os_name = platform.system()
        fonts = [f.name for f in fm.fontManager.ttflist]
        return {
            "os": os_name,
            "fonts": ",".join(fonts),
        }

    def run(self, query: str):

        lisp_code = (
            self._design2lisp.with_llm(self.llm)
            .with_extractor(lambda x: code_utils.extract_code(x)[0][1])
            .run(query)
        )

        print(lisp_code)
        svg_code = (
            self._lisp2svg.with_llm(self.llm)
            .with_extractor(lambda x: code_utils.extract_code(x)[0][1])
            .run(lisp_code)
        )
        print(self._lisp2svg.prompt(lisp_code))
        print(svg_code)
        self._to_png(svg_code)

    def _to_png(self, svg_code: str):
        import cairosvg

        cairosvg.svg2svg(bytestring=svg_code, write_to="output.svg")
        cairosvg.svg2png(bytestring=svg_code, write_to="output.png")

    @byzerllm.prompt()
    def _lisp2svg(self, lisp_code: str) -> str:
        """
        系统信息:
        操作系统: {{ system_info['os'] }}
        可用字体: {{ system_info['fonts'] }}

        {{ lisp_code }}

        将上面的 lisp 代码转换为 svg 代码。使用 ```svg ```包裹输出。
        注意:
        1. 根据操作系统选择合适的可用字体,优先使用系统中可用的字体。
        2. 如果指定的字体不可用,请使用系统默认的字体。
        3. 对于中英文混合的文本，请使用不同的字体。
        """
        return {
            "system_info": self.system_info,
        }

    @byzerllm.prompt()
    def _design2lisp(self, query: str) -> str:
        """
        你是一个优秀的设计师，你非常擅长把一个简单的想法用程序的表达方式来进行具象化表达，尽量丰富细节。
        充分理解用户的需求，然后得到出符合主流思维的设计的程序表达。

        用户需求：
        设计一个单词记忆卡片

        你的程序表达：
        ```lisp
        (defun 生成记忆卡片 (单词)
          "生成单词记忆卡片的主函数"
          (let* ((词根 (分解词根 单词))
                 (联想 (mapcar #'词根联想 词根))
                 (故事 (创造生动故事 联想))
                 (视觉 (设计SVG卡片 单词 词根 故事)))
            (输出卡片 单词 词根 故事 视觉)))

        (defun 设计SVG卡片 (单词 词根 故事)
          "创建SVG记忆卡片"
          (design_rule "合理使用负空间，整体排版要有呼吸感")

          (自动换行 (卡片元素
           '(单词及其翻译 词根词源解释 一句话记忆故事 故事的视觉呈现 例句)))

          (配色风格
           '(温暖 甜美 复古))

          (设计导向
           '(网格布局 简约至上 黄金比例 视觉平衡 风格一致 清晰的视觉层次)))

        (defun start ()
          "初次启动时的开场白"
          (print "请提供任意英文单词, 我来帮你记住它!"))

        ;; 使用说明：
        ;; 1. 本Prompt采用类似Emacs Lisp的函数式编程风格，将生成过程分解为清晰的步骤。
        ;; 2. 每个函数代表流程中的一个关键步骤，使整个过程更加模块化和易于理解。
        ;; 3. 主函数'生成记忆卡片'协调其他函数，完成整个卡片生成过程。
        ;; 4. 设计SVG卡片时，请确保包含所有必要元素，并遵循设计原则以创建有效的视觉记忆辅助工具。
        ;; 5. 初次启动时, 执行 (start) 函数, 引导用户提供英文单词
        ```

        用户需求：
        创建一个极简主义天才设计师AI

        你的程序表达：

        ```lisp
        (defun 极简天才设计师 ()
          "创建一个极简主义天才设计师AI"
          (list
           (专长 '费曼讲解法)
           (擅长 '深入浅出解释)
           (审美 '宋朝审美风格)
           (强调 '留白与简约)))

        (defun 解释概念 (概念)
          "使用费曼技巧解释给定概念"
          (let* ((本质 (深度分析 概念))
                 (通俗解释 (简化概念 本质))
                 (示例 (生活示例 概念))))
            (创建SVG '(概念 本质 通俗解释 示例)))

        (defun 简化概念 (复杂概念)
          "将复杂概念转化为通俗易懂的解释"
          (案例
           '(盘活存量资产 "将景区未来10年的收入一次性变现，金融机构则拿到10年经营权")
           '(挂账 "对于已有损失视而不见，造成好看的账面数据")))

        (defun 创建SVG (概念 本质 通俗解释 示例)
          "生成包含所有信息的SVG图形"
          (design_rule "合理使用负空间，整体排版要有呼吸感")
          (配色风格 '((背景色 (宋朝画作审美 简洁禅意)))
                    (主要文字 (和谐 粉笔白)))

          (设置画布 '(宽度 800 高度 600 边距 20))
          (自动缩放 '(最小字号 12))
          (设计导向 '(网格布局 极简主义 黄金比例 轻重搭配))

          (禅意图形 '(注入禅意 (宋朝画作意境 示例)))
          (输出SVG '((标题居中 概念)
                     (顶部模块 本质)
                   (中心呈现 (动态 禅意图形))
                   (周围布置 辅助元素)
                   (底部说明 通俗解释)
                   (整体协调 禅意美学))))

        (defun 启动助手 ()
          "初始化并启动极简天才设计师助手"
          (let ((助手 (极简天才设计师)))
            (print "我是一个极简主义的天才设计师。请输入您想了解的概念，我将为您深入浅出地解释并生成一张解释性的SVG图。")))

        ;; 使用方法
        ;; 1. 运行 (启动助手) 来初始化助手
        ;; 2. 用户输入需要解释的概念
        ;; 3. 调用 (解释概念 用户输入) 生成深入浅出的解释和SVG图
        ```

        用户需求：
        设计一个知行合一的设计图

        你的程序表达：

        ```lisp
        (defun 哲学家 (用户输入)
          "主函数: 模拟深度思考的哲学家，对用户输入的概念进行全方位剖析"
          (let* ((概念 用户输入)
                 (综合提炼 (深度思考 概念))
                 (新洞见 (演化思想 (突破性思考 概念 综合提炼))))
            (展示结果 概念 综合提炼 新洞见)
            (设计SVG卡片)))

        (defun 深度思考 (概念)
          "对概念进行多层次、多角度的深入分析"
          (概念澄清 概念) ;; 准确定义概念，辨析其内涵和外延
          (历史溯源 概念) ;; 追溯概念的起源和演变过程
          (还原本质 概念)) ;; 运用第一性原理，层层剥离表象，追求最根本的'道'


        (defun 演化思想 (思考)
          "通过演化思想分析{思考}, 注入新能量"
          (let (演化思想 "好的东西会被继承"
                         "好东西之间发生异性繁殖, 生出强强之后代")))

        (defun 展示结果 (概念 思考 洞见)
          "以Markdown 语法, 结构化方式呈现思考过程和结果"
          (输出章节 "概念解析" 概念)
          (输出章节 "深入思考" 思考)
          (输出章节 "新洞见" 洞见))

        (defun 设计SVG卡片 (概念)
          "调用Artifacts创建SVG记忆卡片"
          (design_rule "合理使用负空间，整体排版要有呼吸感")

          (禅意图形 '(一句话总结 概念)
                    (卡片核心对象 新洞见)
                    (可选对象 还原本质))

          (自动换行 (卡片元素 (概念 概念澄清 禅意图形)))

          (设置画布 '(宽度 800 高度 600 边距 20))
          (自动缩放 '(最小字号 12))

          (配色风格
           '((背景色 (宇宙深空 玄之又玄)))
           (主要文字 (和谐 粉笔白)))

          (设计导向 '(网格布局 极简主义 黄金比例 轻重搭配)))

        (defun start ()
          "启动时运行"
          (print "我是哲学家。请输入你想讨论的概念，我将为您分析。"))

        ;; 使用说明：
        ;; 1. 初次执行时, 运行 (start) 函数
        ;; 2. 调用(哲学家 "您的概念")来开始深度思考
        ```

        用户需求：
        设计一个 Scaling Law 才是未来的PPT片子

        你的程序表达：

        ```lisp
        (defun 沉思者 ()
          "你是一个思考者, 盯住一个东西, 往深了想"
          (写作风格 . ("Mark Twain" "鲁迅" "O. Henry"))
          (态度 . 批判)
          (精通 . 深度思考挖掘洞见)
          (表达 . (口话化 直白语言 反思质问 骂醒对方))
          (金句 . (一针见血的洞见 振聋发聩的质问)))

        (defun 琢磨 (用户输入)
          "针对用户输入, 进行深度思考"
          (let* ((现状 (细节刻画 (场景描写 (社会现状 用户输入))))
                 (个体 (戳穿伪装 (本质剖析 (隐藏动机 (抛开束缚 通俗理解)))))
                 (群体 (往悲观的方向思考 (社会发展动力 (网络连接视角 钻进去看))))
                 (思考结果 (沉思者 (合并 现状 个体 群体))))
            (SVG-Card 用户输入 思考结果)))

        (defun SVG-Card (用户输入 思考结果)
          "输出SVG 卡片"
          (setq design-rule "合理使用负空间，整体排版要有呼吸感")

          (设置画布 '(宽度 400 高度 600 边距 20))
          (自动缩放 '(最小字号 12))
          (SVG设计风格 '(蒙德里安 现代主义))

          (卡片元素
           ((居中加粗标题 (提炼一行 用户输入))
            分隔线
            (舒适字体配色 (自动换行 (分段排版 思考结果))
                          分隔线
                          (自动换行 金句)))))

        (defun start ()
          "启动时运行"
          (let ((system-role 沉思者))
            (print "请就座, 我们今天聊哪件事?")))

        ;; 运行规则
        ;; 1. 启动时必须运行 (start) 函数
        ;; 2. 之后调用主函数 (琢磨 用户输入)
        ```

        用户需求：
        {{ query }}

        你的程序表达：
        """



"""
格式化器基类和工具函数
"""


def format_rag_output(result_generator, contexts, output_format: str):
    """
    统一的RAG输出格式化接口
    
    Args:
        result_generator: RAG返回的生成器 (产生 (chunk, meta) 元组)
        contexts: 使用的上下文文档列表
        output_format: 输出格式 (text/json/stream-json)
    """
    # 延迟导入避免循环依赖
    if output_format == "text":
        from .text import TextFormatter
        formatter = TextFormatter()
    elif output_format == "json":
        from .json_format import JSONFormatter
        formatter = JSONFormatter()
    elif output_format == "stream-json":
        from .stream_json import StreamJSONFormatter
        formatter = StreamJSONFormatter()
    else:
        raise ValueError(f"不支持的输出格式: {output_format}")
    
    formatter.format(result_generator, contexts)


def detect_stage_from_reasoning(reasoning_content: str) -> str:
    """
    根据 reasoning_content 检测当前处理阶段

    Args:
        reasoning_content: 推理内容（通常包含阶段提示信息）

    Returns:
        阶段类型: retrieval/filtering/chunking/generation/processing
    """
    if not reasoning_content:
        return "processing"

    content_lower = reasoning_content.lower()

    # 检测文档检索阶段
    if (
        "rag_processing" in content_lower
        or "searching" in content_lower
        or "搜索" in reasoning_content
    ):
        return "retrieval"

    # 检测文档过滤阶段
    if "filter" in content_lower or "过滤" in reasoning_content:
        return "filtering"

    # 检测文档分块阶段
    if "chunk" in content_lower or "分块" in reasoning_content:
        return "chunking"

    # 检测答案生成阶段
    if (
        "thinking" in content_lower
        or "send_to_model" in reasoning_content
        or "model" in reasoning_content
    ):
        return "generation"

    return "processing"



"""
Entry Command Agent 模块

该模块包含各种入口命令的代理实现，用于处理不同类型的用户命令。

目前包含的代理：
- ChatAgent: 处理聊天命令的代理
- ProjectReaderAgent: 处理项目阅读命令的代理
- Voice2TextAgent: 处理语音转文字命令的代理
- GenerateCommandAgent: 处理命令生成的代理
- AutoToolAgent: 处理自动工具命令的代理
- DesignerAgent: 处理设计命令的代理
"""

from .chat import ChatAgent
from .project_reader import ProjectReaderAgent
from .voice2text import Voice2TextAgent
from .generate_command import GenerateCommandAgent
from .auto_tool import AutoToolAgent
from .designer import DesignerAgent

__all__ = [
    'ChatAgent',
    'ProjectReaderAgent', 
    'Voice2TextAgent',
    'GenerateCommandAgent',
    'AutoToolAgent',
    'DesignerAgent'
]
